"""reset_attr_positions

Revision: 0094
Revision ID: 31367f4af05e
Revises: 248a52525842
Create Date: 2017-06-09 10:35:45.010815

"""

# revision identifiers, used by Alembic.
revision = '31367f4af05e'
down_revision = '248a52525842'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.execute("""
        WITH positions as (
            SELECT id, row_number() OVER (partition by group_id, container order by position) AS p
            FROM attributes
        ) UPDATE attributes SET position = positions.p - 1 FROM positions
        WHERE attributes.id = positions.id
    """)

    op.execute("""
        DELETE FROM attribute_positions USING attributes, projects
        WHERE attributes.id = attribute_positions.attribute_id AND attributes.group_id = projects.attribute_group_id
    """)

    op.execute("""
        WITH positions as (
            SELECT ap.id, row_number() OVER (partition by a.group_id, ap.namespace, ap.container, ap.relative_to order by ap.position) AS p
            FROM attribute_positions ap JOIN attributes a ON a.id = ap.attribute_id
        ) UPDATE attribute_positions SET position = positions.p - 1 FROM positions
        WHERE attribute_positions.id = positions.id
    """)


def downgrade():
    pass



"""kantree_special_user_avatar_url

Revision: 0177
Revision ID: 23727a96713b
Revises: 4fc19fa2e6da
Create Date: 2020-03-04 15:15:45.789846

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '23727a96713b'
down_revision = '4fc19fa2e6da'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        UPDATE users SET avatar_filename = replace(avatar_filename, '/static/img/favicon.png', '/static/favicon.png') WHERE username = 'Kantree' AND is_special_user
    """)


def downgrade():
    pass



"""user_invited_by_link

Revision: 0237
Revision ID: f026609b2ae2
Revises: 5f92d1bc0cfa
Create Date: 2022-03-04 11:31:29.867251

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = 'f026609b2ae2'
down_revision = '5f92d1bc0cfa'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('users', sa.Column('invited_by_link', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'users', 'organizations', ['invited_by_link'], ['id'], ondelete='SET NULL')


def downgrade():
    op.drop_column('users', 'invited_by_link')



"""integrations

Revision: 0126
Revision ID: 431110172682
Revises: 2387660ed43b
Create Date: 2017-08-10 17:46:53.622528

"""

# revision identifiers, used by Alembic.
revision = '431110172682'
down_revision = '2387660ed43b'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


def upgrade():
    op.create_table('integrations',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('org_id', sa.Integer(), nullable=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('name', sa.String(), nullable=True),
        sa.Column('account', sa.String(), nullable=True),
        sa.Column('meta', postgresql.JSON(), nullable=True),
        sa.Column('authorized', sa.Boolean(), nullable=True),
        sa.Column('error', sa.String(), nullable=True),
        sa.ForeignKeyConstraint(['org_id'], ['organizations.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_integrations_org_id'), 'integrations', ['org_id'], unique=False)
    op.create_index(op.f('ix_integrations_name'), 'integrations', ['name'], unique=False)

    op.create_table('project_integration_scopes',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('project_id', sa.Integer(), nullable=True),
        sa.Column('integration_id', sa.Integer(), nullable=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('options', postgresql.JSON(), nullable=True),
        sa.Column('webhook_token', sa.String(), nullable=True),
        sa.ForeignKeyConstraint(['project_id'], ['projects.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['integration_id'], ['integrations.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_project_integration_scopes_project_id'), 'project_integration_scopes', ['project_id'], unique=False)
    op.create_index(op.f('ix_project_integration_scopes_integration_id'), 'project_integration_scopes', ['integration_id'], unique=False)

    op.add_column('attributes', sa.Column('is_from_integration', sa.String(), nullable=True))
    op.add_column('attributes', sa.Column('integration_ref', sa.String(), nullable=True))
    op.create_index(op.f('ix_attributes_is_from_integration_integration_ref'), 'attributes', ['is_from_integration', 'integration_ref'], unique=False)

    op.execute("""INSERT INTO integrations (org_id, created_at, created_by_id, name, meta, authorized, error)
        SELECT p.owner_id, cb.activated_at, cb.activated_by_id, cb.name, cb.meta, not cb.needs_reactivation, cb.error
        FROM card_behaviors cb JOIN projects p ON p.id = cb.project_id""")

    op.execute("""INSERT INTO project_integration_scopes (project_id, integration_id, created_at, created_by_id, options)
        SELECT p.id, i.id, cb.enabled_at, cb.enabled_by_id, cb.options::json
        FROM integrations i JOIN projects p ON p.owner_id = i.org_id JOIN card_behaviors cb ON cb.name = i.name AND cb.project_id = p.id""")

    # TEMPORARY
    op.execute("""DELETE FROM integrations WHERE id IN (
        SELECT integrations.id FROM integrations JOIN organizations ON organizations.id = integrations.org_id
        WHERE organizations.is_user_org
    )""")


def downgrade():
    op.drop_table('project_integration_scopes')
    op.drop_table('integrations')
    op.drop_column('attributes', 'is_from_integration')
    op.drop_column('attributes', 'integration_ref')



"""remove_observers_from_orgs

Revision: 0225
Revision ID: f04a85eeb070
Revises: 58cf93323b46
Create Date: 2021-09-23 19:08:42.479572

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = 'f04a85eeb070'
down_revision = '58cf93323b46'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        WITH orgs AS (
            SELECT DISTINCT om.org_id
            FROM organization_members om
            JOIN users u ON om.user_id = u.id
            WHERE u.is_observer IS TRUE
        )
        UPDATE organizations
        SET cache_id = uuid_generate_v4()
        FROM orgs
        WHERE id = orgs.org_id
    """)
    op.execute("""
        DELETE
        FROM organization_members om
        USING users u
        WHERE om.user_id = u.id
        AND u.is_observer IS TRUE
    """)
    op.execute("""
        UPDATE users
        SET perm_cache_id = uuid_generate_v4()
        WHERE is_observer IS TRUE
    """)


def downgrade():
    pass



"""labels_to_groups

Revision: 0041
Revision ID: 5409e2aad044
Revises: 23e5cefde476
Create Date: 2015-11-28 10:04:42.342027

"""

# revision identifiers, used by Alembic.
revision = '5409e2aad044'
down_revision = '23e5cefde476'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.sql import select
from sqlalchemy.dialects import postgresql

card_attributes = sa.Table(
    'card_attributes',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('card_model_attribute_id', sa.Integer),
    sa.Column('card_id', sa.Integer),
    sa.Column('value', postgresql.JSONB)
)
card_model_attributes = sa.Table(
    'card_model_attributes',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('type_name', sa.String),
    sa.Column('name', sa.String),
    sa.Column('container', sa.String),
    sa.Column('is_from_builtin', sa.Boolean),
    sa.Column('card_model_id', sa.Integer),
    sa.Column('options', sa.PickleType)
)
card_models = sa.Table(
    'card_models',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer)
)
cards = sa.Table(
    'cards',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer),
    sa.Column('parent_id', sa.Integer),
    sa.Column('parents_ids', postgresql.ARRAY(sa.Integer), default=list),
    sa.Column('children_attributes_ctx', sa.PickleType, default=dict)
)
card_enabled_group_types = sa.Table(
    'card_enabled_group_types',
    sa.MetaData(),
    sa.Column('group_type_id', sa.Integer),
    sa.Column('card_id', sa.Integer)
)
orgs = sa.Table(
    'organizations',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True)
)
card_group_types = sa.Table(
    'card_group_types',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer),
    sa.Column('name', sa.String),
    sa.Column('singular_name', sa.String),
    sa.Column('plural_name', sa.String),
    sa.Column('scope', sa.String),
    sa.Column('one_group_per_card', sa.Boolean),
    sa.Column('display_on_card', sa.String),
    sa.Column('auto_random_color', sa.Boolean),
    sa.Column('builtin', sa.String),
    sa.Column('auto_enabled_on_new_projects', sa.Boolean),
    sa.Column('is_project_specific', sa.Boolean),
)
card_groups = sa.Table(
    'card_groups',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('parent_id', sa.Integer),
    sa.Column('group_type_id', sa.Integer),
    sa.Column('title', sa.String),
    sa.Column('color', sa.String),
    sa.Column('position', sa.Integer)
)
card_groups_cards = sa.Table(
    'card_groups_cards',
    sa.MetaData(),
    sa.Column('group_id', sa.Integer),
    sa.Column('card_id', sa.Integer),
    sa.Column('position', sa.Integer)
)

def upgrade():
    conn = op.get_bind()

    local_contexts = set()
    needed_groups = {}
    card_labels = {}

    q = select([card_attributes, cards.c.parent_id, cards.c.parents_ids, cards.c.owner_id])\
        .where(sa.and_(
            card_model_attributes.c.id == card_attributes.c.card_model_attribute_id,
            cards.c.id==card_attributes.c.card_id,
            card_model_attributes.c.type_name=='labels',
            card_model_attributes.c.is_from_builtin==True,
            cards.c.owner_id!=None
        ))
    for attr in conn.execute(q):
        if not attr.parent_id:
            root_id = attr.card_id
            ctx_id = attr.card_id
        else:
            root_id = attr.parents_ids[0]
            ctx_id = attr.parent_id
        if root_id not in needed_groups:
            needed_groups[root_id] = [{}, attr.owner_id]
        if ctx_id not in local_contexts:
            local_contexts.add(ctx_id)
            ctx = conn.execute(select([cards.c.children_attributes_ctx])
                .where(cards.c.id==ctx_id)).scalar()
            for title, color in ctx.get('labels:labels', {}).items():
                needed_groups[root_id][0].setdefault(title, color)
        if attr.value:
            for title in attr.value:
                needed_groups[root_id][0].setdefault(title, None)
            card_labels[attr.card_id] = (attr.value, root_id)

    gtypes = {}
    for org in conn.execute(select([orgs.c.id])):
        r = conn.execute(card_group_types.insert().values(
            owner_id=org.id,
            name='Labels',
            singular_name='label',
            plural_name='labels',
            scope='project',
            one_group_per_card=False,
            display_on_card='border',
            auto_random_color=True,
            builtin='LABELS',
            auto_enabled_on_new_projects=True,
            is_project_specific=False
        ))
        gtypes[org.id] = r.inserted_primary_key[0]
        conn.execute(card_enabled_group_types.insert().from_select(['group_type_id', 'card_id'],
            select([sa.text(str(gtypes[org.id])), cards.c.id]).where(
                sa.and_(cards.c.parent_id==None, cards.c.owner_id==org.id))))

        conn.execute(card_model_attributes.update().where(sa.and_(
            card_models.c.id==card_model_attributes.c.card_model_id,
            card_models.c.owner_id==org.id,
            card_model_attributes.c.type_name=='labels',
            card_model_attributes.c.name=='labels',
            card_model_attributes.c.container==None
        )).values(
            type_name='group_type',
            options={'group_type': gtypes[org.id]}
        ))

    groups = {}
    for id, spec in needed_groups.items():
        groups[id] = {}
        type_id = gtypes[spec[1]]
        for title, color in spec[0].items():
            r = conn.execute(card_groups.insert().values(
                parent_id=id,
                group_type_id=type_id,
                title=title,
                color=color,
                position=len(groups[id].items())))
            groups[id][title] = r.inserted_primary_key[0]

    positions = {}
    values = []
    for id, spec in card_labels.items():
        for title in spec[0]:
            gid = groups[spec[1]][title]
            values.append({'card_id': id, 'group_id': gid, 'position': positions.get(gid, 0)})
            positions[gid] = positions.get(gid, 0) + 1
    op.bulk_insert(card_groups_cards, values)

    op.execute("""DELETE FROM card_attributes USING card_model_attributes
            WHERE card_model_attributes.id = card_attributes.card_model_attribute_id AND
            card_model_attributes.is_from_builtin AND card_attributes.type_name = 'labels'""")


def downgrade():
    pass



"""prevent_org_deletion

Revision: 0189
Revision ID: 5bcdd81f9309
Revises: 68d34c581b25
Create Date: 2020-08-14 09:23:35.452231

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '5bcdd81f9309'
down_revision = '68d34c581b25'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('organizations', sa.Column('prevent_deletion', sa.Boolean(), nullable=True))
    op.execute("UPDATE organizations SET prevent_deletion=false")


def downgrade():
    op.drop_column('organizations', 'prevent_deletion')



"""activities_project_id

Revision: 0082
Revision ID: 3ce9106c2680
Revises: 4ae4624010e9
Create Date: 2017-01-13 10:22:40.928396

"""

# revision identifiers, used by Alembic.
revision = '3ce9106c2680'
down_revision = '4ae4624010e9'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('activities', sa.Column('project_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'activities', 'projects', ['project_id'], ['id'], ondelete='SET NULL')
    op.create_index(op.f('ix_activities_project_id'), 'activities', ['project_id'], unique=False)

    op.drop_index(op.f('ix_activities_object_id'), 'activities')
    op.drop_index(op.f('ix_activities_object_type'), 'activities')
    op.create_index(op.f('ix_activities_object_id_object_type'), 'activities', ['object_id', 'object_type'], unique=False)

    op.drop_index(op.f('ix_activities_sub_object_id'), 'activities')
    op.drop_index(op.f('ix_activities_sub_object_type'), 'activities')
    op.create_index(op.f('ix_activities_sub_object_id_sub_object_type'), 'activities', ['sub_object_id', 'sub_object_type'], unique=False)

    op.drop_index(op.f('ix_activities_target_id'), 'activities')
    op.drop_index(op.f('ix_activities_target_type'), 'activities')
    op.create_index(op.f('ix_activities_target_id_target_type'), 'activities', ['target_id', 'target_type'], unique=False)

    op.drop_index(op.f('ix_activities_origin_id'), 'activities')
    op.drop_index(op.f('ix_activities_origin_type'), 'activities')
    op.create_index(op.f('ix_activities_origin_id_origin_type'), 'activities', ['origin_id', 'origin_type'], unique=False)

    op.create_index(op.f('ix_activities_verb'), 'activities', ['verb'], unique=False)
    op.create_index(op.f('ix_activities_created_by_id'), 'activities', ['created_by_id'], unique=False)

    op.execute("""UPDATE activities SET project_id = cards.project_id FROM cards, card_groups
        WHERE cards.id = card_groups.parent_id AND card_groups.parent_id is not null
            AND card_groups.id = activities.object_id AND activities.object_type = 'CardGroup'""")

    op.execute("""UPDATE activities SET project_id = cards.project_id FROM cards, card_groups
        WHERE cards.id = card_groups.parent_id AND card_groups.parent_id is not null
            AND card_groups.id = activities.target_id AND activities.target_type = 'CardGroup'""")

    op.execute("""UPDATE activities SET project_id = request_forms.project_id FROM requests, request_forms
        WHERE request_forms.id = requests.request_form_id AND requests.id = activities.target_id
            AND activities.target_type = 'Request'""")

    op.execute("""UPDATE activities SET project_id = cards.project_id FROM cards
        WHERE cards.id = activities.object_id AND activities.object_type = 'Card'""")

    op.execute("""UPDATE activities SET project_id = cards.project_id FROM cards
        WHERE cards.id = activities.target_id AND activities.target_type = 'Card'""")


def downgrade():
    op.drop_index(op.f('ix_activities_project_id'), 'activities')
    op.drop_column('activities', 'project_id')



"""view_position

Revision: 0081
Revision ID: 4ae4624010e9
Revises: 30ad9d5dfff6
Create Date: 2017-02-06 22:02:12.685926

"""

# revision identifiers, used by Alembic.
revision = '4ae4624010e9'
down_revision = '30ad9d5dfff6'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('views', sa.Column('position', sa.Integer(), nullable=True))
    op.create_index(op.f('ix_views_position'), 'views', ['position'], unique=False)
    op.execute("""
        WITH positions as (
            SELECT id, row_number() OVER (partition by project_id order by id) AS p
            FROM views
        ) UPDATE views SET position = positions.p - 1 FROM positions
        WHERE positions.id = views.id
    """)


def downgrade():
    op.drop_index(op.f('ix_views_position'), table_name='views')
    op.drop_column('views', 'position')



"""migrate_rels_attr_projects_options_value

Revision ID: 0253
Revises: 0252
Create Date: 2023-06-23 11:17:10.998559

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0253'
down_revision = '0252'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        UPDATE attributes
        SET options = jsonb_strip_nulls(jsonb_set(jsonb_set(options, '{project}', jsonb_path_query_first(options, '$.projects.id')), '{projects}', 'null'::jsonb))
        WHERE type_name = 'relationships' AND options ? 'projects'
    """)


def downgrade():
    pass



# Application title
TITLE = "Kantree"

# IMPORTANT: key to encrypt the session
SECRET_KEY = "changeme"


# ---------------------------------------------------------------------------------------------------------
# SERVER

# IMPORTANT: public hostname from which Kantree is accessible
SERVER_NAME = "localhost:5000"

# Application server port
SERVER_PORT = 5000

# Database URI (PostgreSQL only)
# See https://docs.sqlalchemy.org/en/latest/core/engines.html#database-urls
DATABASE_URI = None

# Redis URI
# See https://redis-py.readthedocs.io/en/latest/#redis.Redis.from_url
REDIS_URI = "redis://localhost:6379/0"

# Whether the server is hosted behind HTTPS
SERVER_SECURED = False

# URL to reach the push server from the browser.
# By default, the push server will be available on port 0.0.0.0:4000
# If you are using a reverse proxy to serve Kantree under a single domain, you will have to
# set this URL
PUSH_SERVER_URL = None

# Port on which to serve the push server
PUSH_SERVER_PORT = 4000

# Secret key for the push server. Will use SECRET_KEY if None
PUSH_SECRET = None

# SMTP settings
SMTP_SERVER = '127.0.0.1'
SMTP_USERNAME = None
SMTP_PASSWORD = None
SMTP_PORT = 25
SMTP_USE_TLS = False

# Default sender for all emails
MAIL_DEFAULT_SENDER = "Kantree (no-reply) <no-reply@kantree.io>"

# Directory where to store uploaded files
UPLOAD_DIR = 'uploads'

# Default locale for users (see kantree/translations for available locales)
DEFAULT_LOCALE = 'en'

# Try to auto detect the user locale based on his browser information
AUTO_DETECT_USER_LOCALE = True

# Force the Service Worker Cache name (by default will use the current version of Kantree).
# Allows to force cache invalidation even when the version does not change
CACHE_SERVICE_WORKER_NAME = None

# Whether to disable completly the cache service worker
DISABLE_CACHE_SERVICE_WORKER = False


# ---------------------------------------------------------------------------------------------------------
# QUEUES & SPECIFIC CONNECTIONS

QUEUE_PROJECT_HOOKS = None
QUEUE_AUTOMATIONS = None
QUEUE_AUTOMATIONS_HIGH_COMPLEXITY = None
QUEUE_RELATED_CARDS = None
QUEUE_COMPUTE_FORMULAS = None
QUEUE_REFRESH_CHARTS = None
QUEUE_BATCH_ACTIONS = None
QUEUE_ASSISTANT = None
QUEUE_PROJECT_IMPORT = None

# Threshold above which an automation is considered of high complexity
AUTOMATION_COMPLEXITY_HIGH_THRESHOLD = 100

# Number of cards to process per batches when computing formulas. Batches are ran in parallel
FORMULA_QUEUE_PARALLEL_BATCH_SIZE = 200


# ---------------------------------------------------------------------------------------------------------
# LOGGING

# Log level (one of DEBUG, INFO, WARNING, ERROR)
LOG_LEVEL = 'INFO'

# Log to file
LOG_FILENAME = None

# Log to syslog (SYSLOG_PORT can also be set)
SYSLOG_HOST = None

# Log exceptions to Sentry (https://sentry.io)
SENTRY_DSN = None

# Sentry tracing sample rate
SENTRY_TRACES_SAMPLE_RATE = 0.05

# Sentry tracing sample rate
SENTRY_PROFILES_SAMPLE_RATE = 0.5

# Display a log file in the admin
ADMIN_LOG_FILENAME = None

# Display an iframe in the admin with RQ dashboard proxied through Kantree
ADMIN_RQ_DASHBOARD_URL = None

# Email address to contact when some limits are reached
ADMIN_WARNING_CONTACT = None

# Whether to log or not the server name mismatch error
IGNORE_SERVER_NAME_MISMTATCH_ERROR = False


# ---------------------------------------------------------------------------------------------------------
# SECURITY

# Enable Two Factor Authentification
ENABLE_2FA = True

# Whether admins can add new users from the admin area
ADMIN_CAN_ADD_NEW_USERS = True

# Whether admins can access all projects without limitations
ADMIN_CAN_ACCESS_ALL_PROJECTS = True

# Whether user can signup. Set to false if using external auth like LDAP
ALLOW_SIGNUP = True

# Whether username can be modified (even by admin). Useful when using external auth like LDAP
READONLY_USERNAME = False

# Whether email can be modified (even by admin)
READONLY_EMAIL = False

# Whether user can reset their password
USER_CAN_RESET_PASSWORD = True

# Whether user can delete their account by themselves
USER_CAN_DELETE_ACCOUNT = True

# Only allow emails from these domains when signing up
NEW_USERS_LIMIT_DOMAINS = []

# Prevent emails using these domains to signup
NEW_USERS_BLOCK_DOMAINS = []

# When signing up, all new users will be defined as observers
NEW_USERS_AS_OBSERVERS = False

# Email domains which will be considered approved
USERS_APPROVED_DOMAINS = []

# When NEW_USERS_AS_OBSERVERS=True, automatically set users matching the approved domain list as full members
NEW_USERS_AS_OBSERVERS_AUTO_APPROVE = True

# Whether projects can be made accessible to visitors without authentification
ALLOW_PUBLIC_PROJECTS = True

# Hide logout button. Useful when using SSO
HIDE_LOGOUT_BUTTON = False

# Show a logged out page instead of redirecting to /
SHOW_LOGGED_OUT_PAGE = False

# Add Content-Security-Policy header
CSP_HEADER = True

# Allowed sources
CSP_SAFE_URLS = ["'self'", "'unsafe-inline'", "'unsafe-eval'"]

# Only allowed embedding using iframes from the listed parents (see CSP frame-ancestors) (None to disable)
IFRAME_ALLOWED_ANCESTORS = ["'self'"]

# Disable the possibility for users to get API keys
DISABLE_USER_API_KEYS = False

# Whether to send an email to ask for validation
VALIDATE_USER_EMAILS = False

# Whether to lock the user out until his email is validated
BLOCK_NON_EMAIL_VALIDATED_USERS = False

# TTL for automation rules run token
AUTOMATION_RULES_RUN_TOKEN_TTL = None

# Whether org members can be invited via a link
ENABLE_ORG_INVITE_LINK = True


# ---------------------------------------------------------------------------------------------------------
# Appearance

# Override the navbar logo (200x50px)
NAVBAR_LOGO_URL = None

# Logo to display on authentification pages (100x100px)
AUTH_PAGE_LOGO = None

# Custom background (css property background) for auth page left panel
AUTH_BACKGROUND = None

# Default interface color
THEME_COLOR = None

DEFAULT_FORM_HEADER_COLOR = None

DEFAULT_COLOR_PALETTE = None


# ---------------------------------------------------------------------------------------------------------
# Signup

# Show this message of the signup page
SIGNUP_MESSAGE = None

# Show the "subscribe to newsletter" option on signup page
SIGNUP_NEWSLETTER = False

# Whether to send a welcome email on signup (kantree/emails/welcome.md)
SIGNUP_WELCOME_EMAIL = False

# Show "Login with SSO" link on login & signup page
SSO_LOGIN_URL = None


# ---------------------------------------------------------------------------------------------------------
# Users settings

# Try getting the avatar from gravatar
USER_TRY_GRAVATAR = False

# Whether user can create personal projects outside of orgs
USER_CAN_CREATE_PROJECTS = True

# Whether user can create private personal projects
USER_CAN_CREATE_PRIVATE_PROJECTS = True

# Whether user can join any organizations
USER_CAN_JOIN_ORGANIZATIONS = True

# Whether user can create orgs by themselves
USER_CAN_CREATE_ORGANIZATIONS = True


# ---------------------------------------------------------------------------------------------------------
# ORGS/TEAMS SETTINGS

# What roles are authorized as guests in projects
# If True, all roles are allowed
# Otherwise, it must be a list of role names from the builtins
ORG_AUTHORIZED_GUEST_ROLES = True

# If set to false, only public projects will be allowed in orgs
ORG_CAN_CREATE_PRIVATE_PROJECTS = True

# Whether orgs require at least one owner.
# By default, orgs with no owners are allowed. In this case, only kantree admins can manage them.
ORG_AT_LEAST_ONE_OWNER_REQUIRED = False

# Whether organization owners can export the org by themselves
ORG_CAN_SELF_EXPORT = False

# Default feature flags to activate hidden features on all orgs
# DEFAULT_FEATURE_FLAGS = ['teams', 'org_hub', 'sso', 'portfolios', 'gantt', 'quadrant']
DEFAULT_FEATURE_FLAGS = ['teams', 'org_hub', 'sso']


# ---------------------------------------------------------------------------------------------------------
# ANTI-SPAM

# Maximum number of invites the user can send at once
MAX_INVITES_AT_ONCE = None

# Maximum number of invites a project/team can have during their lifetime
MAX_TOTAL_INVITES = None

# Maximum pending invites in a project or team
MAX_PENDING_INVITES = 100

# Use recaptcha validation on signup
RECAPTCHA_KEY = None
RECAPTCHA_SECRET = None

# Use hcaptcha validation on signup
HCAPTCHA_KEY = None
HCAPTCHA_SECRET = None

# Maximum number of email templates a team can send per day (None to disable)
MAX_EMAIL_TEMPLATE_SEND_PER_DAY = None


# ---------------------------------------------------------------------------------------------------------
# TASK TIMEOUTS

# Timeout (in seconds) for import tasks, can be increased when importing large projects
PROJECT_IMPORTER_TASK_TIMEOUT = 500

# Timeout (in seconds) for tasks that refresh chart data
REFRESH_CHART_TASK_TIMEOUT = 300

# Timeout (in seconds) for automations
AUTOMATIONS_TASK_TIMEOUT = 180


# ---------------------------------------------------------------------------------------------------------
# TEMPLATE LIBRARY

# Template library accessibe to unauthenticated users
# Note: even if public, project not marked as public won't be visible to unauthenticated users
PUBLIC_TEMPLATE_LIBRARY = False

# Only display the following tags in the template library.
# Other tags are still permitted but only visible on the template itself.
# Can be a dict for localized tags where keys are the locale name.
TEMPLATE_LIBRARY_DISPLAY_TAGS = None

# Whether any users can publish templates to the library
OPEN_TEMPLATE_LIBRARY = True

# Default template (id of a project)
DEFAULT_TEMPLATE_ID = None

# Name of the tag used for featured templates.
# Can be a dict for localized tag where keys are the locale name.
TEMPLATE_LIBRARY_FEATURED_TAG = 'Featured'

# Hide selection of the preferred template library tag during onboarding
ONBOARDING_HIDE_PREFERRED_TEMPLATE_TAG = False


# ---------------------------------------------------------------------------------------------------------
# HELP & ONBOARDING

# Link towards your custom help site (kantree.io used otherwise)
HELP_SITE_URL = None

# Display a button to open support tickets (opens the url in a new tab)
HELP_TICKET_URL = None

# Show this email for people who want to contact support
SUPPORT_EMAIL = None

# Slug of a project which will be automatically copied to all new users (users must be allowed to have private projects)
ONBOARDING_WELCOME_PROJECT = None

# Name of an org/team to automatically join when signing up
AUTO_JOIN_ORG_AFTER_SIGNUP = None

# Show onboarding tour to new users
ONBOARDING_TOUR_ENABLED = True

# Whether a multi-day drip email campaign to users during their onboarding process
SEND_ONBOARDING_EMAILS = False

# Automatically delete orgs when all members are removed
ORG_AUTO_DELETE_WHEN_NO_MEMBERS = False

# Whether to automatically disable users after no access for X days
AUTO_DISABLE_USER_AFTER_DAYS_NOT_ACCESSED = False

# Show the what's new button
WHATSNEW_ID = 3


# ---------------------------------------------------------------------------------------------------------
# Legal

# Path to a file containing the Terms & Conditions
TERMS_FILE = None

# URL of a T&C page to display a link in the interface
TERMS_URL = None

# Last update date of the Terms & Conditions (YYYY-MM-DD).
# If set, users who haven't accepted terms past this date will be prompted to accept T&C
TERMS_UPDATE_DATE = None

# URL to a privacy page
PRIVACY_URL = None

# URL to data processing agreement page
DATA_PROCESSING_AGREEMENT_URL = None

# Whether to show the terms validation popup after signup
FORCE_TERMS_VALIDATION_AFTER_SIGNUP = False


# ---------------------------------------------------------------------------------------------------------
# AI


# Provider to use (one of "OpenAI", "Claude", "Mistral")
AI_PROVIDER = "Mistral"

# Use a custom provider:
# AI_PROVIDER = {
#     "base_url": "https://api.mistral.ai/v1",
#     "model": "mistral-medium-latest",
#     "kql_model": "mistral-medium-latest", # optional, model dedicated to generating KQL
#     "agent_model": "mistral-medium-latest", # optional, model dedicated to agents
# }

# API KEY
AI_API_KEY = None

# Whether orgs automatically have AI features enabled when globally configured
# If false, AI will be enabled only for some features. Additional features like
# Kantree Assistant and AI automations will required each org to provide an API key.
AI_ORG_AUTO_ENABLED = True

# Whether the assistant can be used
AI_ASSISTANT = True


# ---------------------------------------------------------------------------------------------------------
# MISC

# Use a different upload backend for exports. You must first configure the upload backend as part of the frasco_upload config
EXPORTS_UPLOAD_BACKEND = None

# Max upload size in bytes
MAX_UPLOAD_SIZE = None

# Max upload size from public forms in bytes
MAX_PUBLIC_UPLOAD_SIZE = None

# Limit file types from public forms
# List with any from: documents, media, images, videos, audio, archives
LIMIT_PUBLIC_UPLOAD_FILE_TYPES = None

# Whether exporting can be done sync instead of async
ALLOW_STREAMING_EXPORTS = True

# What action to do when synchronizing users via csv (one of: observer, disable, delete)
BULK_IMPORT_USERS_SYNC = 'disable'

# Domain that will be used for inbound emails and for the reply-to field of outgoing notifications
EMAIL_INBOUND_DOMAIN = None

# Emails to ignore in CCs field when handling inbound emails
IGNORE_INBOUND_EMAIL_CC = []

# When the user is not logged in, the homepage should display the content of this file
PROMO_FILE = None

# When the user is not logged in, the homepage should redirect to the following URL
PROMO_REDIRECT = None

# Same as before but a dict where keys are the locale name and values the url to redirect to
PROMO_LOCALE_REDIRECT = {}

# Track signups with GA (the script will only be included on /signup and /login)
GOOGLE_ANALYTICS_ID = None

# Track signups with Plausible (the script will only be included on /signup and /login)
PLAUSIBLE_URL = None

# Whether to allow the demo parameter on the template lib and projects
ENABLE_DEMO_MODE = False

# Token needed to download geoip databases
MAXMIND_GEOIP_TOKEN = None

KANTREE_COACH_ENABLED = False

# Token to your kantree coach service portal
KANTREE_COACH_TOKEN = None



"""kql_flat

Revision: 0145
Revision ID: 2e10b78f004e
Revises: 3bcdc4dfe14c
Create Date: 2018-10-17 15:56:21.370847

"""

# revision identifiers, used by Alembic.
revision = '2e10b78f004e'
down_revision = '3bcdc4dfe14c'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.execute("""
        CREATE OR REPLACE FUNCTION kql_flat(data integer[]) RETURNS integer
        AS
        $BODY$
            SELECT data[1];
        $BODY$
        LANGUAGE sql;
    """)


    op.execute("""
        CREATE OR REPLACE FUNCTION kql_flat(data numeric[]) RETURNS numeric
        AS
        $BODY$
            SELECT data[1];
        $BODY$
        LANGUAGE sql;
    """)


    op.execute("""
        CREATE OR REPLACE FUNCTION kql_flat(data text[]) RETURNS text
        AS
        $BODY$
            SELECT data[1];
        $BODY$
        LANGUAGE sql;
    """)


    op.execute("""
        CREATE OR REPLACE FUNCTION kql_flat(data text) RETURNS text
        AS
        $BODY$
            SELECT data;
        $BODY$
        LANGUAGE sql;
    """)


    op.execute("""
        CREATE OR REPLACE FUNCTION kql_flat(data integer) RETURNS integer
        AS
        $BODY$
            SELECT data;
        $BODY$
        LANGUAGE sql;
    """)


    op.execute("""
        CREATE OR REPLACE FUNCTION kql_flat(data numeric) RETURNS numeric
        AS
        $BODY$
            SELECT data;
        $BODY$
        LANGUAGE sql;
    """)


def downgrade():
    op.execute("DROP FUNCTION kql_flat(integer[])")
    op.execute("DROP FUNCTION kql_flat(numeric[])")
    op.execute("DROP FUNCTION kql_flat(text[])")
    op.execute("DROP FUNCTION kql_flat(integer)")
    op.execute("DROP FUNCTION kql_flat(numeric)")
    op.execute("DROP FUNCTION kql_flat(text)")



"""project_last_activity_at

Revision: 0174
Revision ID: ace95229377
Revises: 10e15642295c
Create Date: 2019-09-22 21:35:23.681042

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = 'ace95229377'
down_revision = '10e15642295c'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('projects', sa.Column('last_activity_at', sa.DateTime(), nullable=True))
    op.execute("UPDATE projects SET last_activity_at = cards.last_activity_at FROM cards WHERE cards.id = projects.top_level_card_id")


def downgrade():
    op.drop_column('projects', 'last_activity_at')



"""Convert location attribute values from string to dict format

Revision ID: 0284
Revises: 0283
Create Date: 2025-12-18

"""

from alembic import op
import sqlalchemy as sa
import json


# revision identifiers, used by Alembic.
revision = "0284"
down_revision = "0283"
branch_labels = None
depends_on = None


def convert_to_dict(value):
    """Convert a location value to the new dict format."""
    if value is None:
        return None

    # Try to parse as JSON first
    if isinstance(value, str):
        try:
            parsed = json.loads(value)
            if isinstance(parsed, list):
                parsed = parsed[0]
            if isinstance(parsed, dict) and "address" in parsed:
                # Already in new format
                return parsed
            # It was a JSON string (e.g., "\"123 Main St\"")
            if isinstance(parsed, str):
                return {"address": parsed, "lat": None, "lng": None}
        except (json.JSONDecodeError, TypeError):
            pass
        # Plain string address
        return {"address": value, "lat": None, "lng": None}

    if isinstance(value, dict):
        if "address" in value:
            return value
        return {"address": str(value), "lat": None, "lng": None}

    return {"address": str(value), "lat": None, "lng": None}


def upgrade():
    conn = op.get_bind()

    # Find all location attribute values
    result = conn.execute(
        sa.text("""
        SELECT id, value
        FROM card_attributes
        WHERE type_name = 'location' AND value IS NOT NULL
    """)
    )

    updates = []
    for row in result:
        new_value = convert_to_dict(row[1])
        updates.append({"value": json.dumps(new_value), "id": row[0]})

    if updates:
        conn.execute(
            sa.text("UPDATE card_attributes SET value = :value WHERE id = :id"),
            updates
        )


def downgrade():
    pass



"""reporting_views

Revision: 0172
Revision ID: 34c57eb46c8a
Revises: 595a3ffb3b2
Create Date: 2019-08-28 10:04:42.691654

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '34c57eb46c8a'
down_revision = '595a3ffb3b2'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        CREATE OR REPLACE VIEW users_report_view AS
        SELECT
            id,
            signup_at,
            signup_from,
            signup_provider,
            auth_providers,
            last_login_at,
            last_login_from,
            last_login_provider,
            split_part(email, '@', 2) email_domain,
            can_create_organizations,
            locale,
            timezone,
            enable_email_notifications,
            updated_at,
            is_admin,
            subscription_required,
            trial_ends_at,
            can_join_organizations,
            is_observer,
            is_disabled,
            disabled_at,
            last_activity_at,
            onboarded,
            last_password_change_at,
            last_terms_validation,
            email_newsletter,
            email_notifications,
            is_super_admin,
            signup_country,
            signup_cpc_source,
            signup_cpc_campaign,
            signup_cpc_ad,
            signup_cpc_keyword,
            signup_cpc_sessionid,
            last_access_at,
            last_password_change_from,
            is_auto_disabled,
            two_factor_auth_enabled
        FROM users
    """)

    op.execute("""
        CREATE OR REPLACE VIEW projects_report_view AS
        SELECT
            p.id,
            p.created_at,
            p.updated_at,
            p.owner_id,
            p.created_by_id,
            p.is_template,
            p.is_archived,
            p.org_members_can_observe,
            p.org_members_can_autojoin,
            p.default_org_member_permission,
            p.import_status,
            p.total_invites,
            p.default_org_member_role_id,
            p.is_public,
            p.send_emails,
            p.allow_anon_inbound_emails,
            p.include_in_template_library,
            p.copied_from_template_id,
            c.last_activity_at,
            (SELECT COUNT(*) FROM project_members WHERE project_id = p.id) nb_members,
            (SELECT COUNT(*) FROM cards WHERE project_id = p.id) nb_cards,
            (SELECT MAX(last_activity_at) FROM cards WHERE project_id = p.id) last_card_activity_at
        FROM projects p JOIN cards c ON c.id = p.top_level_card_id
    """)

    op.execute("""
        CREATE OR REPLACE VIEW organizations_report_view AS
        SELECT
            id,
            created_at,
            created_by_id,
            name,
            can_invite_guest_collabs,
            trial_ends_at,
            updated_at,
            slug,
            billing_country,
            eu_vat_country,
            has_stripe_source,
            plan_last_charge_amount,
            plan_last_charge_successful,
            plan_last_charged_at,
            plan_name,
            plan_next_charge_at,
            plan_status,
            stripe_customer_id,
            stripe_subscription_id,
            company,
            subscription_required,
            payment_method,
            can_create_private_projects,
            can_create_projects,
            total_invites,
            is_prepaid,
            prepaid_until,
            max_allowed_users,
            can_self_export,
            email_templates_send_counter,
            (SELECT string_agg(email_domains.d, ', ') FROM (SELECT DISTINCT split_part(u.email, '@', 2) d FROM users u JOIN organization_members om ON om.user_id = u.id WHERE om.org_id = organizations.id AND om.permission = 4) email_domains) "owner_email_domains",
            (SELECT COUNT(*) FROM organization_members WHERE org_id = organizations.id) nb_members,
            (SELECT COUNT(*) FROM projects WHERE owner_id = organizations.id AND NOT is_archived AND NOT is_template) nb_projects,
            (SELECT COUNT(*) FROM projects WHERE owner_id = organizations.id AND is_archived AND NOT is_template) nb_archived_projects,
            (SELECT COUNT(*) FROM projects WHERE owner_id = organizations.id AND is_template) nb_templates
        FROM organizations
        WHERe NOT is_user_org
    """)


def downgrade():
    op.execute("DROP VIEW users_report_view")
    op.execute("DROP VIEW projects_report_view")
    op.execute("DROP VIEW organizations_report_view")



"""models_cache_id

Revision: 0018
Revision ID: 355b90f95fc4
Revises: 26dc21c88e6
Create Date: 2015-08-17 14:45:44.630343

"""

# revision identifiers, used by Alembic.
revision = '355b90f95fc4'
down_revision = '26dc21c88e6'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('card_models', sa.Column('cache_id', sa.String(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('card_models', 'cache_id')
    ### end Alembic commands ###



"""dettach_github_sync_attrs

Revision: 0240
Revision ID: 130b5c21c344
Revises: 3fa23c3f6c07
Create Date: 2022-05-04 22:32:26.437893

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '130b5c21c344'
down_revision = '3fa23c3f6c07'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        UPDATE attributes SET is_from_integration = null WHERE is_from_integration = 'github' AND integration_ref IN ('labels', 'assignees', 'milestone')
    """)


def downgrade():
    pass



"""activity_username

Revision: 0113
Revision ID: 4d7f1ad39d84
Revises: edc4da1f645
Create Date: 2018-02-27 11:07:15.183763

"""

# revision identifiers, used by Alembic.
revision = '4d7f1ad39d84'
down_revision = 'edc4da1f645'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('activities', sa.Column('created_by_username', sa.String(), nullable=True))
    op.add_column('card_comments', sa.Column('created_by_username', sa.String(), nullable=True))
    op.add_column('card_logs', sa.Column('created_by_username', sa.String(), nullable=True))
    op.add_column('card_change_history', sa.Column('created_by_username', sa.String(), nullable=True))

    op.execute("""
        UPDATE activities
        SET created_by_username = users.username
        FROM users
        WHERE users.id = activities.created_by_id
    """)
    op.execute("""
        UPDATE card_comments
        SET created_by_username = users.username
        FROM users
        WHERE users.id = card_comments.created_by_id
    """)
    op.execute("""
        UPDATE card_logs
        SET created_by_username = users.username
        FROM users
        WHERE users.id = card_logs.created_by_id
    """)
    op.execute("""
        UPDATE card_change_history
        SET created_by_username = users.username
        FROM users
        WHERE users.id = card_change_history.created_by_id
    """)


def downgrade():
    op.drop_column('activities', 'created_by_username')
    op.drop_column('card_comments', 'created_by_username')
    op.drop_column('card_logs', 'created_by_username')
    op.drop_column('card_change_history', 'created_by_username')



"""view_categories

Revision: 0234
Revision ID: 249b8b1c2bd6
Revises: 437b60ce2a7c
Create Date: 2021-11-30 16:31:08.580092

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '249b8b1c2bd6'
down_revision = '437b60ce2a7c'
branch_labels = None
depends_on = None


def upgrade():
    op.create_table('view_categories',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('project_id', sa.Integer(), nullable=True),
        sa.Column('name', sa.String(), nullable=False),
        sa.Column('position', sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['project_id'], ['projects.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_view_categories_project_id'), 'view_categories', ['project_id'], unique=False)

    op.add_column('views', sa.Column('category_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'views', 'view_categories', ['category_id'], ['id'], ondelete='SET NULL')


def downgrade():
    op.drop_column('views', 'category_id')
    op.drop_table('view_categories')



"""project_guide

Revision: 0098
Revision ID: 3a512b4f3d8f
Revises: 4e63e5c80ada
Create Date: 2017-06-20 18:17:57.849503

"""

# revision identifiers, used by Alembic.
revision = '3a512b4f3d8f'
down_revision = '4e63e5c80ada'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('projects', sa.Column('guide', sa.Text(), nullable=True))


def downgrade():
    op.drop_column('projects', 'guide')



"""invoice_eu_vat_amount

Revision: 0028
Revision ID: 43ed95babe0a
Revises: 13c9e3e73486
Create Date: 2015-09-22 20:03:26.698797

"""

# revision identifiers, used by Alembic.
revision = '43ed95babe0a'
down_revision = '13c9e3e73486'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    op.add_column('invoices', sa.Column('eu_vat_amount', sa.Float(), nullable=True))


def downgrade():
    op.drop_column('invoices', 'eu_vat_amount')



"""add mongo_id columns for migration

Revision: 0002
Revision ID: 8c4d6851928
Revises: 34eb10cee620
Create Date: 2015-06-03 20:27:44.740583

"""

# revision identifiers, used by Alembic.
revision = '8c4d6851928'
down_revision = '34eb10cee620'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('board_templates', sa.Column('mongo_id', sa.String(length=24), nullable=True))
    op.add_column('card_lists', sa.Column('mongo_id', sa.String(length=24), nullable=True))
    op.add_column('card_models', sa.Column('mongo_id', sa.String(length=24), nullable=True))
    op.add_column('cards', sa.Column('mongo_id', sa.String(length=24), nullable=True))
    op.add_column('milestones', sa.Column('mongo_id', sa.String(length=24), nullable=True))
    op.add_column('organizations', sa.Column('mongo_id', sa.String(length=24), nullable=True))
    op.add_column('users', sa.Column('mongo_id', sa.String(length=24), nullable=True))
    op.create_index(op.f('ix_users_mongo_id'), 'users', ['mongo_id'], unique=False)
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f('ix_users_mongo_id'), table_name='users')
    op.drop_column('users', 'mongo_id')
    op.drop_column('organizations', 'mongo_id')
    op.drop_column('milestones', 'mongo_id')
    op.drop_column('cards', 'mongo_id')
    op.drop_column('card_models', 'mongo_id')
    op.drop_column('card_lists', 'mongo_id')
    op.drop_column('board_templates', 'mongo_id')
    ### end Alembic commands ###



"""migrate_view_modes

Revision: 0218
Revision ID: 681ff6dac033
Revises: bac8354bc42a
Create Date: 2021-10-22 16:20:31.251297

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '681ff6dac033'
down_revision = 'bac8354bc42a'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("UPDATE views SET mode = 'report' WHERE mode = 'analytics'")


def downgrade():
    pass



"""import_export_role

Revision: 0169
Revision ID: 21cd415ba55b
Revises: 2212e3bc7f5f
Create Date: 2019-07-03 18:36:57.078155

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '21cd415ba55b'
down_revision = '2212e3bc7f5f'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        UPDATE role_permissions SET actions = array_append(actions, 'IMPORT_DATA')
        FROM roles WHERE roles.id = role_permissions.role_id
        AND roles.builtin = 'ADMIN'
        AND role_permissions.object_type = 'project'
        AND role_permissions.object_id IS NULL
        AND NOT (role_permissions.actions @> '{"IMPORT_DATA"}'::varchar[])
    """)

    op.execute("""
        UPDATE role_permissions SET actions = array_append(actions, 'EXPORT_DATA')
        FROM roles WHERE roles.id = role_permissions.role_id
        AND roles.builtin = 'ADMIN'
        AND role_permissions.object_type = 'project'
        AND role_permissions.object_id IS NULL
        AND NOT (role_permissions.actions @> '{"EXPORT_DATA"}'::varchar[])
    """)

    op.execute("""
        UPDATE roles SET cache_id = uuid_generate_v4()
        WHERE roles.id IN (
            SELECT DISTINCT roles.id
            FROM roles, role_permissions
            WHERE role_permissions.role_id = roles.id
            AND roles.builtin = 'ADMIN'
            AND role_permissions.object_type = 'project'
            AND role_permissions.object_id IS NULL
            AND (role_permissions.actions @> '{"IMPORT_DATA"}'::varchar[])
        )
    """)

    op.execute("""
        UPDATE projects SET members_cache_id = uuid_generate_v4() FROM organizations, roles, role_permissions
        WHERE organizations.id = projects.owner_id
            AND roles.owner_id = organizations.id
            AND roles.builtin = 'ADMIN'
            AND role_permissions.role_id = roles.id
            AND role_permissions.object_type = 'project'
            AND role_permissions.object_id IS NULL
            AND (role_permissions.actions @> '{"IMPORT_DATA"}'::varchar[])
    """)


def downgrade():
    pass



"""scope_card_group_position

Revision: 0135
Revision ID: 49d68aabe6a1
Revises: 11fab9822bf6
Create Date: 2018-06-28 12:01:16.701330

"""

# revision identifiers, used by Alembic.
revision = '49d68aabe6a1'
down_revision = '11fab9822bf6'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.execute("""
        WITH positions as (
            SELECT group_id, card_id, row_number() OVER (partition by group_id, cards.parent_id order by position) AS p
            FROM card_groups_cards JOIN cards ON cards.id = card_groups_cards.card_id
            WHERE NOT cards.is_archived
        ) UPDATE card_groups_cards SET position = positions.p - 1 FROM positions
        WHERE positions.group_id = card_groups_cards.group_id AND positions.card_id = card_groups_cards.card_id
    """)

    op.execute("""
        UPDATE cards SET cache_id = uuid_generate_v4() FROM card_groups_cards cgt WHERE cgt.card_id = cards.id
    """)

    op.execute("""
        WITH positions as (
            SELECT id, row_number() OVER (partition by group_type_id, parent_id order by position) AS p
            FROM card_groups WHERE NOT is_archived
        ) UPDATE card_groups SET position = positions.p - 1 FROM positions
        WHERE positions.id = card_groups.id
    """)


def downgrade():
    pass



"""flag_card_template_sub_cards

Revision: 0241
Revision ID: 6372d988531f
Revises: 130b5c21c344
Create Date: 2022-06-07 19:15:53.636735

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '6372d988531f'
down_revision = '130b5c21c344'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        WITH template_cards AS (
            SELECT *
            FROM cards AS c
            WHERE c.is_template = True
        )
        UPDATE cards AS c
        SET is_template = True
        FROM template_cards AS tc WHERE tc.id = ANY (c.parents_ids)
    """)


def downgrade():
    pass



"""relationships_v2

Revision: 0204
Revision ID: e2464817d291
Revises: 397b2cb08fff
Create Date: 2021-01-28 09:40:47.009138

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.sql import select
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = 'e2464817d291'
down_revision = '397b2cb08fff'
branch_labels = None
depends_on = None


attributes = sa.Table(
    'attributes',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('type_name', sa.String),
    sa.Column('options', sa.PickleType)
)

cards = sa.Table(
    'cards',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('project_id', sa.Integer),
    sa.Column('parent_id', sa.Integer),
    sa.Column('title', sa.String)
)


def upgrade():
    op.execute("""
        UPDATE card_attributes
        SET value = jsonb_path_query_array(value, '$[*] ? (@.reversed == false).connected_card')
        WHERE type_name = 'relationships'
    """)

    op.execute("""
        DELETE FROM card_attributes
        WHERE type_name = 'relationships' AND jsonb_array_length(value) = 0
    """)

    conn = op.get_bind()
    q = select([attributes]).where(attributes.c.type_name == 'card')
    for attr in conn.execute(q):
        if not attr.options or not attr.options.get('card'):
            continue
        card = conn.execute(select([cards]).where(cards.c.id==attr.options['card'])).fetchone()
        if card and not card.parent_id:
            kql = 'project = "%s"' % card.title
        else:
            kql = 'parent = %s' % (card.id if card else attr.options['card'])
        conn.execute(attributes.update().where(attributes.c.id==attr.id).values(options={'search': kql}))

    op.execute("UPDATE cards SET cache_id = generate_cache_id(id) WHERE id IN (SELECT card_id FROM card_attributes WHERE type_name='relationships')")



def downgrade():
    pass



"""recurring_tasks

Revision: 0110
Revision ID: 1b7722f7b357
Revises: 50312a20c5ea
Create Date: 2018-01-12 10:50:50.264648

"""

revision = '1b7722f7b357'
down_revision = '50312a20c5ea'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    op.create_table('recurring_tasks',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('project_id', sa.Integer(), nullable=True),
    sa.Column('template_id', sa.Integer(), nullable=True),
    sa.Column('created_by_id', sa.Integer(), nullable=True),
    sa.Column('created_at', sa.DateTime(), nullable=True),
    sa.Column('updated_at', sa.DateTime(), nullable=True),
    sa.Column('recurring_days', sa.String(), nullable=True),
    sa.Column('recurring_time', sa.Time(), nullable=True),
    sa.Column('last_creation_at', sa.DateTime(), nullable=True),
    sa.ForeignKeyConstraint(['template_id'], ['cards.id'], ondelete='CASCADE'),
    sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
    sa.ForeignKeyConstraint(['project_id'], ['projects.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('recurring_task_groups',
    sa.Column('task_id', sa.Integer(), nullable=True),
    sa.Column('group_id', sa.Integer(), nullable=True),
    sa.ForeignKeyConstraint(['group_id'], ['card_groups.id'], ondelete='CASCADE'),
    sa.ForeignKeyConstraint(['task_id'], ['recurring_tasks.id'], ondelete='CASCADE')
    )
    op.create_index(op.f('ix_recurring_task_groups_group_id'), 'recurring_task_groups', ['group_id'], unique=False)
    op.create_index(op.f('ix_recurring_task_groups_task_id'), 'recurring_task_groups', ['task_id'], unique=False)
    op.create_index(op.f('ix_recurring_tasks_template_id'), 'recurring_tasks', ['template_id'], unique=False)
    op.create_index(op.f('ix_recurring_tasks_project_id'), 'recurring_tasks', ['project_id'], unique=False)
    cards = sa.Table('cards', sa.MetaData(), sa.Column('is_template', sa.Boolean))
    op.execute(cards.update().values(**{'is_template': False}))


def downgrade():
    op.execute("DELETE FROM cards WHERE is_template = TRUE")
    op.drop_index(op.f('ix_recurring_task_groups_task_id'), table_name='recurring_task_groups')
    op.drop_index(op.f('ix_recurring_task_groups_group_id'), table_name='recurring_task_groups')
    op.drop_table('recurring_task_groups')
    op.drop_index(op.f('ix_recurring_tasks_project_id'), table_name='recurring_tasks')
    op.drop_index(op.f('ix_recurring_tasks_template_id'), table_name='recurring_tasks')
    op.drop_table('recurring_tasks')



"""locked_structure

Revision ID: 0264
Revises: 0263
Create Date: 2025-02-05 13:16:17.623883

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0264'
down_revision = '0263'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('projects', sa.Column('locked_structure', sa.Boolean(), nullable=False, server_default="false"))


def downgrade():
    op.drop_column('projects', 'locked_structure')


"""card_toolbars

Revision ID: 0278
Revises: 0277
Create Date: 2025-02-28 11:02:20.935912

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql
import datetime

# revision identifiers, used by Alembic.
revision = '0278'
down_revision = '0277'
branch_labels = None
depends_on = None


def upgrade():
    op.create_table('card_toolbars',
        sa.Column('id', sa.Integer, primary_key=True),
        sa.Column('created_at', sa.DateTime, default=datetime.datetime.utcnow),
        sa.Column('updated_at', sa.DateTime, default=datetime.datetime.utcnow),
        sa.Column('project_id', sa.Integer, sa.ForeignKey('projects.id', ondelete='CASCADE')),
        sa.Column('model_id', sa.Integer, sa.ForeignKey('card_models.id', ondelete='CASCADE')),
    )
    op.create_index(op.f('ix_card_toolbars_project_id_model_id'), 'card_toolbars', ['project_id', 'model_id'], unique=False)

    op.create_table('card_toolbar_automation_rules',
        sa.Column('toolbar_id', sa.Integer, sa.ForeignKey('card_toolbars.id', ondelete='CASCADE')),
        sa.Column('rule_id', sa.Integer, sa.ForeignKey('automation_rules.id', ondelete='CASCADE')),
        sa.Column('position', sa.Integer, nullable=False, server_default="0"),
        sa.PrimaryKeyConstraint('toolbar_id', 'rule_id'),
    )
    op.create_index(op.f('ix_card_toolbar_automation_rules_toolbar_id'), 'card_toolbar_automation_rules', ['toolbar_id'], unique=False)


def downgrade():
    op.drop_table('card_toolbar_automation_rules')
    op.drop_table('card_toolbars')




"""oauth

Revision ID: 0268
Revises: 0267
Create Date: 2025-06-17 13:16:17.623883

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0268'
down_revision = '0267'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('oauth_authz_codes', sa.Column('acr', sa.Text()))
    op.add_column('oauth_authz_codes', sa.Column('amr', sa.Text()))
    op.add_column('oauth_app_tokens', sa.Column('access_token_revoked_at', sa.Integer(), nullable=False, server_default='0'))
    op.add_column('oauth_app_tokens', sa.Column('refresh_token_revoked_at', sa.Integer(), nullable=False, server_default='0'))


def downgrade():
    op.drop_column('oauth_authz_codes', 'acr')
    op.drop_column('oauth_authz_codes', 'amr')
    op.drop_column('oauth_app_tokens', 'access_token_revoked_at')
    op.drop_column('oauth_app_tokens', 'refresh_token_revoked_at')


"""request_form_add_to_parent

Revision: 0151
Revision ID: 3124afe3927f
Revises: 476d0278b0a7
Create Date: 2018-11-30 15:56:57.172795

"""

# revision identifiers, used by Alembic.
revision = '3124afe3927f'
down_revision = '476d0278b0a7'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('request_forms', sa.Column('add_to_parent_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'request_forms', 'cards', ['add_to_parent_id'], ['id'], ondelete='SET NULL')


def downgrade():
    op.drop_column('request_forms', 'add_to_parent_id')



"""delete_no_perm_role

Revision ID: 0275
Revises: 0274
Create Date: 2025-08-26 11:55:09.675251

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0275'
down_revision = '0274'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
               DELETE FROM project_members WHERE role_id is null
            """)


def downgrade():
    pass



"""deduplicate_ref

Revision: 0154
Revision ID: 64703a6228b
Revises: 25a07f58df10
Create Date: 2018-12-13 16:28:06.429052

"""

# revision identifiers, used by Alembic.
revision = '64703a6228b'
down_revision = '25a07f58df10'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    conn = op.get_bind()
    q = sa.text("select id, project_id, ref from cards c1 where (select count(*) from cards c2 where c2.ref = c1.ref and c2.project_id = c1.project_id) > 1")

    refs_per_project = {}
    for r in conn.execute(q):
        refs = refs_per_project.setdefault(r[1], [])
        if r[2] not in refs:
            refs.append(r[2])
        else:
            new_ref = conn.execute(sa.text("UPDATE projects SET ref_counter = ref_counter + 1 WHERE id = %s RETURNING ref_counter" % r[1])).scalar()
            op.execute("UPDATE cards SET ref = %s, cache_id = uuid_generate_v4() WHERE id = %s" % (new_ref, r[0]))



def downgrade():
    pass



"""user_timezone

Revision: 0227
Revision ID: cec3b3150599
Revises: b069c965f0d3
Create Date: 2021-11-29 10:11:08.355866

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = 'cec3b3150599'
down_revision = 'b069c965f0d3'
branch_labels = None
depends_on = None


user_time_format = sa.Enum('am_pm', '24h', name='user_time_format')


def upgrade():
    op.add_column('users', sa.Column('dow_start', sa.Integer(), nullable=True))
    user_time_format.create(op.get_bind(), checkfirst=False)
    op.add_column('users', sa.Column('time_format', user_time_format, nullable=True))


def downgrade():
    op.drop_column('users', 'dow_start')
    op.drop_column('users', 'time_format')
    user_time_format.drop(op.get_bind(), checkfirst=False)



"""inbox

Revision: 0208
Revision ID: 41d379ba09af
Revises: 0f6cbf29afd9
Create Date: 2021-02-26 16:02:57.613574

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '41d379ba09af'
down_revision = '0f6cbf29afd9'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        DELETE FROM activities
        WHERE object_type = 'CardComment' AND NOT EXISTS (SELECT 1 FROM card_comments WHERE card_comments.id = activities.object_id)
    """)


def downgrade():
    pass



"""card_group_type_display_on_cards

Revision: 0030
Revision ID: 11c328aaee9c
Revises: 25e88bf7445c
Create Date: 2015-11-03 21:05:05.038813

"""

# revision identifiers, used by Alembic.
revision = '11c328aaee9c'
down_revision = '25e88bf7445c'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('card_group_types', sa.Column('singular_name', sa.String(), nullable=True))
    op.add_column('card_group_types', sa.Column('plural_name', sa.String(), nullable=True))
    op.add_column('card_group_types', sa.Column('icon', sa.String(), nullable=True))
    op.add_column('card_group_types', sa.Column('display_on_card', sa.Boolean(), nullable=True))
    op.execute("UPDATE card_group_types SET display_on_card = true, icon = 'flag-checkered', singular_name = 'milestone', plural_name = 'milestones' WHERE name = 'Milestones'")
    op.execute("UPDATE card_group_types SET display_on_card = false, singular_name = 'list', plural_name = 'lists' WHERE name = 'Lists'")


def downgrade():
    op.drop_column('card_group_types', 'singular_name')
    op.drop_column('card_group_types', 'plural_name')
    op.drop_column('card_group_types', 'icon')
    op.drop_column('card_group_types', 'display_on_cards')



"""remove_dependencies_field

Revision ID: 0280
Revises: 0279
Create Date: 2019-02-19 22:34:16.800414

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '0280'
down_revision = '0279'
branch_labels = None
depends_on = None


def upgrade():
    # Delete problematic card_attributes rows before updating
    op.execute("""
        DELETE FROM card_attributes
        WHERE type_name='relationships'
          AND (value = 'null'::jsonb
               OR jsonb_array_length(value) = 0
               OR value IS NULL)
    """)

    op.execute("""
        UPDATE card_attributes
        SET value = (
            SELECT jsonb_agg(jsonb_build_array(elem::integer, 'none', 'end_start'))
            FROM jsonb_array_elements_text(value) AS elem
        )
        WHERE type_name='relationships'
    """)

    op.execute("""
        UPDATE card_attributes
        SET type_name = 'relationships'
        WHERE type_name='dependencies'
    """)

    op.execute("""
        UPDATE attributes
        SET type_name = 'relationships'
        WHERE type_name='dependencies'
    """)


def downgrade():
    pass



"""missing_kql_flat

Revision: 0150
Revision ID: 476d0278b0a7
Revises: 49495b7b574
Create Date: 2018-11-26 14:54:22.734393

"""

# revision identifiers, used by Alembic.
revision = '476d0278b0a7'
down_revision = '49495b7b574'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.execute("""
        CREATE OR REPLACE FUNCTION kql_flat(data date[]) RETURNS date
        AS
        $BODY$
            SELECT data[1];
        $BODY$
        LANGUAGE sql;
    """)


    op.execute("""
        CREATE OR REPLACE FUNCTION kql_flat(data date) RETURNS date
        AS
        $BODY$
            SELECT data;
        $BODY$
        LANGUAGE sql;
    """)


def downgrade():
    op.execute("DROP FUNCTION kql_flat(date[])")
    op.execute("DROP FUNCTION kql_flat(date)")



"""log_type_to_org

Revision: 0090
Revision ID: 4be9e24293aa
Revises: 6dac66a4490
Create Date: 2017-03-17 17:48:06.314584

"""

# revision identifiers, used by Alembic.
revision = '4be9e24293aa'
down_revision = '6dac66a4490'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('card_log_types', sa.Column('owner_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'card_log_types', 'organizations', ['owner_id'], ['id'], ondelete='CASCADE')
    op.create_index(op.f('ix_card_log_types_owner_id'), 'card_log_types', ['owner_id'], unique=False)

    op.add_column('card_log_types', sa.Column('is_project_specific', sa.Boolean(), nullable=True))

    op.create_table(
        'project_enabled_log_types',
        sa.Column('log_type_id', sa.Integer(), nullable=False),
        sa.Column('project_id', sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(['project_id'], ['projects.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['log_type_id'], ['card_log_types.id'], ondelete='CASCADE')
    )
    op.create_index(op.f('ix_project_enabled_log_types_log_type_id'), 'project_enabled_log_types', ['log_type_id'], unique=False)
    op.create_index(op.f('ix_project_enabled_log_types_project_id'), 'project_enabled_log_types', ['project_id'], unique=False)

    op.execute("INSERT INTO project_enabled_log_types (project_id, log_type_id) SELECT project_id, id FROM card_log_types")
    op.execute("UPDATE card_log_types SET is_project_specific = true")

    op.drop_column('card_log_types', 'project_id')


def downgrade():
    op.drop_column('card_log_types', 'is_project_specific')
    op.drop_column('card_log_types', 'owner_id')



"""group_always_show_desc

Revision: 0095
Revision ID: 19a97b706e5a
Revises: 31367f4af05e
Create Date: 2017-06-18 17:19:29.809802

"""

# revision identifiers, used by Alembic.
revision = '19a97b706e5a'
down_revision = '31367f4af05e'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('card_groups', sa.Column('always_show_description', sa.Boolean(), nullable=True))


def downgrade():
    op.drop_column('card_groups', 'always_show_description')



"""total_invites

Revision: 0100
Revision ID: 3b0d34ece0d5
Revises: 439d72ec8dc4
Create Date: 2017-07-18 13:49:44.968381

"""

# revision identifiers, used by Alembic.
revision = '3b0d34ece0d5'
down_revision = '439d72ec8dc4'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('organizations', sa.Column('total_invites', sa.Integer(), nullable=True))
    op.add_column('projects', sa.Column('total_invites', sa.Integer(), nullable=True))
    op.execute("UPDATE organizations SET total_invites = (SELECT COUNT(*) FROM organization_members om WHERE om.org_id = organizations.id)")
    op.execute("UPDATE projects SET total_invites = (SELECT COUNT(*) FROM project_members pm WHERE pm.project_id = projects.id)")


def downgrade():
    op.drop_column('organizations', 'total_invites')
    op.drop_column('projects', 'total_invites')



"""user_app_params

Revision: 0144
Revision ID: 3bcdc4dfe14c
Revises: 10f9a04dddda
Create Date: 2018-09-28 18:18:36.842151

"""

# revision identifiers, used by Alembic.
revision = '3bcdc4dfe14c'
down_revision = '10f9a04dddda'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


def upgrade():
    op.add_column('users', sa.Column('app_params', postgresql.JSON(), nullable=True))
    op.execute("UPDATE users SET app_params='{\"theme\":\"topbar\"}'::json")


def downgrade():
    op.drop_column('users', 'app_params')



"""models without builtin, listable

Revision: 0037
Revision ID: 11af45c6c79e
Revises: 52ccef8b0e0
Create Date: 2015-11-26 16:28:18.436640

"""

# revision identifiers, used by Alembic.
revision = '11af45c6c79e'
down_revision = '52ccef8b0e0'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import datetime
import uuid
from sqlalchemy.dialects import postgresql

orgs_table = sa.Table(
    'organizations',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('default_card_model_id', sa.Integer),
    sa.Column('is_user_org', sa.Boolean)
)

card_models_table = sa.Table(
    'card_models',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer),
    sa.Column('is_base', sa.String),
    sa.Column('name', sa.String),
    sa.Column('extends_id', sa.Integer),
    sa.Column('created_in_id', sa.Integer),
    sa.Column('created_at', sa.DateTime),
    sa.Column('auto_enabled_on_new_projects', sa.Boolean),
    sa.Column('has_builtin_attributes', sa.Boolean),
    sa.Column('dynamic_extends', sa.String),
    sa.Column('is_project_specific', sa.String),
    sa.Column('cache_id', sa.String)
)


card_model_attributes_table = sa.Table(
    'card_model_attributes',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('card_model_id', sa.Integer),
    sa.Column('position', sa.Integer),
    sa.Column('name', sa.String),
    sa.Column('title', sa.String),
    sa.Column('type_name', sa.String),
    sa.Column('container', sa.String),
    sa.Column('is_from_builtin', sa.Boolean)
)

cards_table = sa.Table(
    'cards',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer),
    sa.Column('model_id', sa.Integer),
    sa.Column('base_card_model_id', sa.Integer),
    sa.Column('parent_id', sa.Integer),
    sa.Column('parents_ids', postgresql.ARRAY(sa.Integer)),
    sa.Column('default_child_card_model_id', sa.Integer),
    sa.Column('title', sa.String)
)

card_enabled_models_table = sa.Table(
    'card_enabled_models',
    sa.MetaData(),
    sa.Column('model_id', sa.Integer),
    sa.Column('card_id', sa.Integer)
)

def upgrade():
    op.add_column('card_model_attributes', sa.Column('is_from_builtin', sa.BOOLEAN(), server_default="False", nullable=False))
    op.drop_column('card_models', 'listable')
    op.add_column('card_models', sa.Column('is_project_specific', sa.BOOLEAN(), server_default="False", nullable=False))
    op.execute("UPDATE card_models SET is_project_specific=true WHERE created_in_id IS NOT NULL AND auto_enabled_on_new_projects = false")
    op.add_column('card_models', sa.Column('has_builtin_attributes', sa.BOOLEAN(), server_default="False", nullable=False))

    op.execute("DELETE FROM cards WHERE owner_id IS NULL")
    conn = op.get_bind()
    for org in conn.execute(orgs_table.select()):
        needOrgModel = True
        needDesc = needAssign = needDate = needLabel = needAttach = True

        org_model_id = org.default_card_model_id
        org_model = conn.execute(card_models_table.select().where(sa.and_(card_models_table.c.owner_id == org.id, card_models_table.c.is_base == 'org'))).first()
        if org_model:
            org_model_id = org_model.id
            needOrgModel = False
            for attr in conn.execute(card_model_attributes_table.select().where(card_model_attributes_table.c.card_model_id == org_model.id)):
                if attr.name == 'description' and attr.type_name == 'content':
                    needDesc = False
                if attr.name == 'assignees' and attr.type_name == 'members':
                    needAssign = False
                if attr.name == 'due_date' and attr.type_name == 'date':
                    needDate = False
                if attr.name == 'labels' and attr.type_name == 'labels':
                    needLabel = False
                if attr.name == 'attachments' and attr.type_name == 'files':
                    needAttach = False

        if needOrgModel:
            res = conn.execute(
                    card_models_table.insert().values(
                        owner_id=org.id,
                        name='User base model' if org.is_user_org else 'Organization base model',
                        auto_enabled_on_new_projects=True,
                        is_base='org',
                        created_at=datetime.datetime.utcnow(),
                        has_builtin_attributes=True,
                        cache_id=str(uuid.uuid4())))
            org_model_id = res.inserted_primary_key[0]

        op.execute("UPDATE organizations SET default_card_model_id = " + str(org_model_id) + " WHERE id = " + str(org.id))

        if needDesc:
            conn.execute(card_model_attributes_table.insert().values(card_model_id=org_model_id, position=0, name='description', title='Description', type_name='content', container='top', is_from_builtin=True))
        if needAssign:
            conn.execute(card_model_attributes_table.insert().values(card_model_id=org_model_id, position=1, name='assignees', title='Assignees', type_name='members', is_from_builtin=True))
        if needDate:
            conn.execute(card_model_attributes_table.insert().values(card_model_id=org_model_id, position=2, name='due_date', title='Due Date', type_name='date', is_from_builtin=True))
        if needLabel:
            conn.execute(card_model_attributes_table.insert().values(card_model_id=org_model_id, position=3, name='labels', title='Labels', type_name='labels', is_from_builtin=True))
        if needAttach:
            conn.execute(card_model_attributes_table.insert().values(card_model_id=org_model_id, position=4, name='attachments', title='Attachments', type_name='files', is_from_builtin=True))

        for card in conn.execute(cards_table.select().where(sa.and_(cards_table.c.owner_id == org.id, cards_table.c.parent_id == None))):
            model_id = card.model_id
            needProjectModel = 'create' if model_id == 1 else 'nothing'
            if needProjectModel == 'nothing':
                needProjectModel = True
                model = conn.execute(card_models_table.select().where(card_models_table.c.id == model_id)).first()
                if not model or model.is_base == 'org':
                    needProjectModel = 'create'
                elif not model.is_base:
                    needProjectModel = 'update'

            if needProjectModel == 'create':
                child = conn.execute(cards_table.select().where(cards_table.c.parent_id == card.id)).first()
                if child and child.model_id != 1:
                    child_model = conn.execute(card_models_table.select().where(card_models_table.c.id == child.model_id)).first()
                    if child_model.extends_id == 1:
                        model_id = child_model.id
                        conn.execute(card_models_table.update().where(card_models_table.c.id == model_id).values(
                            auto_enabled_on_new_projects=False,
                            is_base='project',
                            created_in_id=card.id,
                            extends_id=None,
                            dynamic_extends='org',
                            is_project_specific=True
                        ))
                        op.execute("UPDATE cards SET model_id = " + str(model_id) + ", base_card_model_id = " + str(model_id) + ", default_child_card_model_id = " + str(model_id) + " WHERE id = " + str(card.id))
                if model_id == card.model_id:
                    res = conn.execute(
                            card_models_table.insert().values(
                                owner_id=org.id,
                                name='%s base model' % card.title,
                                auto_enabled_on_new_projects=False,
                                is_base='project',
                                created_in_id=card.id,
                                created_at=datetime.datetime.utcnow(),
                                dynamic_extends='org',
                                is_project_specific=True,
                                cache_id=str(uuid.uuid4())))
                    model_id = res.inserted_primary_key[0]
                    op.execute("UPDATE cards SET model_id = " + str(model_id) + ", base_card_model_id = " + str(model_id) + ", default_child_card_model_id = " + str(model_id) + " WHERE id = " + str(card.id))
                    op.execute("UPDATE cards SET model_id = " + str(model_id) + " WHERE parents_ids[1] = " + str(card.id))
            if needProjectModel == 'update':
                dyn = "null" if model.extends_id is None else "'org'"
                op.execute("UPDATE card_models SET is_base='project', auto_enabled_on_new_projects=false, extends_id=null, dynamic_extends=" + dyn + ", is_project_specific=true, created_in_id=" + str(card.id) + " WHERE id = " + str(card.model_id))
            op.execute("UPDATE cards SET model_id = " + str(model_id) + " WHERE parents_ids[1] = " + str(card.id) + " AND model_id=1")

            res = conn.execute(card_enabled_models_table.select().where(sa.and_(card_enabled_models_table.c.model_id == org_model_id, card_enabled_models_table.c.card_id == card.id))).first()
            if res is None:
                op.execute("INSERT INTO card_enabled_models(model_id, card_id) VALUES (" + str(org_model_id) + ", " + str(card.id) +")")
            res = conn.execute(card_enabled_models_table.select().where(sa.and_(card_enabled_models_table.c.model_id == model_id, card_enabled_models_table.c.card_id == card.id))).first()
            if res is None:
                op.execute("INSERT INTO card_enabled_models(model_id, card_id) VALUES (" + str(model_id) + ", " + str(card.id) +")")

    op.execute("UPDATE cards AS c SET model_id=c2.model_id FROM cards AS c2 WHERE c.parents_ids[1]=c2.id AND (c.model_id=1 OR c.model_id IS NULL)")
    op.execute("UPDATE card_models SET extends_id = null WHERE is_base = 'org' AND extends_id = 1")
    op.execute("UPDATE card_models SET extends_id = null, dynamic_extends = 'org' WHERE extends_id = 1")
    op.execute("UPDATE card_models SET extends_id = null, dynamic_extends = 'org' FROM organizations WHERE extends_id = organizations.default_card_model_id")
    op.execute("UPDATE card_models AS cm SET extends_id = null, dynamic_extends = 'project' FROM card_models AS cm2 WHERE cm.extends_id = cm2.id AND cm2.is_base = 'project'")
    op.execute("UPDATE card_models SET children_default_model_id = null WHERE children_default_model_id = 1")
    op.execute("INSERT INTO card_enabled_models (model_id, card_id) SELECT model_id, parents_ids[1] FROM cards where parents_ids IS NOT NULL")
    op.execute("DELETE FROM card_enabled_models WHERE ctid NOT IN (SELECT min(ctid) FROM card_enabled_models GROUP BY model_id, card_id);")
    op.execute("DELETE FROM card_enabled_models WHERE model_id = 1")

    op.execute("UPDATE cards SET parents_ids = '{13925,27417}' where parent_id = 27417")
    op.execute("UPDATE cards SET parents_ids = '{13925,27417}' where parent_id = 22813")


def downgrade():
    op.drop_column('card_models', 'has_builtin_attributes')
    op.drop_column('card_model_attributes', 'is_from_builtin')
    op.add_column('card_models', sa.Column('listable', sa.BOOLEAN(), server_default="False", nullable=False))
    op.execute("UPDATE card_models SET listable = true WHERE is_base IS NULL AND is_project_specific = false AND auto_enabled_on_new_projects = true")
    op.drop_column('card_models', 'is_project_specific')
    op.execute("UPDATE card_models SET extends_id = 1 WHERE is_base = 'org'")
    op.execute("DELETE FROM card_model_attributes cma USING card_models cm WHERE cm.id = cma.card_model_id AND cm.extends_id = 1 AND cma.name in ('description', 'assignees', 'due date', 'labels', 'attachments')")



"""reindex_positions

Revision: 0045
Revision ID: 34d8e77407
Revises: 21e56a0093b
Create Date: 2015-12-23 21:34:41.956525

"""

# revision identifiers, used by Alembic.
revision = '34d8e77407'
down_revision = '21e56a0093b'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.execute("""
        WITH positions as (
            SELECT group_id, card_id, row_number() OVER (partition by group_id order by position) AS p
            FROM card_groups_cards JOIN cards ON cards.id = card_groups_cards.card_id
            WHERE NOT cards.is_archived
        ) UPDATE card_groups_cards set position = positions.p - 1 FROM positions
        WHERE positions.group_id = card_groups_cards.group_id AND positions.card_id = card_groups_cards.card_id
    """)

    op.execute("""
        WITH positions as (
            SELECT id, row_number() OVER (partition by group_type_id, parent_id order by position) AS p
            FROM card_groups WHERE NOT is_archived
        ) UPDATE card_groups set position = positions.p - 1 FROM positions
        WHERE positions.id = card_groups.id
    """)


def downgrade():
    pass



"""behaviors

Revision: 0020
Revision ID: 51ef836d17f
Revises: 74bc667256f
Create Date: 2015-08-31 15:30:37.277910

"""

# revision identifiers, used by Alembic.
revision = '51ef836d17f'
down_revision = '74bc667256f'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('card_behaviors', sa.Column('activated_at', sa.DateTime(), nullable=True))
    op.add_column('card_behaviors', sa.Column('activated_by_id', sa.Integer(), nullable=True))
    op.add_column('card_behaviors', sa.Column('enabled', sa.Boolean(), nullable=True))
    op.add_column('card_behaviors', sa.Column('meta', postgresql.JSON(), nullable=True))
    op.add_column('card_behaviors', sa.Column('needs_reactivation', sa.Boolean(), nullable=True))
    op.create_foreign_key(None, 'card_behaviors', 'users', ['activated_by_id'], ['id'])
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, 'card_behaviors', type_='foreignkey')
    op.drop_column('card_behaviors', 'needs_reactivation')
    op.drop_column('card_behaviors', 'meta')
    op.drop_column('card_behaviors', 'enabled')
    op.drop_column('card_behaviors', 'activated_by_id')
    op.drop_column('card_behaviors', 'activated_at')
    ### end Alembic commands ###



"""added unindexed_value

Revision: 0005
Revision ID: 22bacef12338
Revises: 4ef62b5814cd
Create Date: 2015-06-05 11:15:40.074153

"""

# revision identifiers, used by Alembic.
revision = '22bacef12338'
down_revision = '4ef62b5814cd'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('card_attributes', sa.Column('unindexed_value', sa.Text(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('card_attributes', 'unindexed_value')
    ### end Alembic commands ###



"""card_group_subs_user_id_index

Revision: 0214
Revision ID: 2e07de14ad8c
Revises: 8db3ad545942
Create Date: 2021-06-02 10:57:49.294543

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '2e07de14ad8c'
down_revision = '8db3ad545942'
branch_labels = None
depends_on = None


def upgrade():
    op.create_index(None, 'card_group_subscriptions', ['user_id'], unique=False)


def downgrade():
    pass



"""Add updated_at and last_activity_at fields

Revision: 0001
Revision ID: 34eb10cee620
Revises: d45177564c9
Create Date: 2015-06-03 15:21:22.002183

"""

# revision identifiers, used by Alembic.
revision = '34eb10cee620'
down_revision = 'd45177564c9'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('board_templates', sa.Column('created_at', sa.DateTime(), nullable=True))
    op.add_column('board_templates', sa.Column('created_by_id', sa.Integer(), nullable=True))
    op.add_column('board_templates', sa.Column('updated_at', sa.DateTime(), nullable=True))
    op.create_foreign_key(None, 'board_templates', 'users', ['created_by_id'], ['id'])
    op.add_column('card_attributes', sa.Column('updated_at', sa.DateTime(), nullable=True))
    op.add_column('card_lists', sa.Column('updated_at', sa.DateTime(), nullable=True))
    op.add_column('card_members', sa.Column('updated_at', sa.DateTime(), nullable=True))
    op.add_column('card_models', sa.Column('created_at', sa.DateTime(), nullable=True))
    op.add_column('card_models', sa.Column('updated_at', sa.DateTime(), nullable=True))
    op.add_column('cards', sa.Column('last_activity_at', sa.DateTime(), nullable=True))
    op.add_column('cards', sa.Column('updated_at', sa.DateTime(), nullable=True))
    op.create_index(op.f('ix_cards_last_activity_at'), 'cards', ['last_activity_at'], unique=False)
    op.add_column('milestones', sa.Column('updated_at', sa.DateTime(), nullable=True))
    op.add_column('organizations', sa.Column('updated_at', sa.DateTime(), nullable=True))
    op.add_column('organization_members', sa.Column('updated_at', sa.DateTime(), nullable=True))
    op.add_column('users', sa.Column('updated_at', sa.DateTime(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('users', 'updated_at')
    op.drop_column('organization_members', 'updated_at')
    op.drop_column('organizations', 'updated_at')
    op.drop_column('milestones', 'updated_at')
    op.drop_index(op.f('ix_cards_last_activity_at'), table_name='cards')
    op.drop_column('cards', 'updated_at')
    op.drop_column('cards', 'last_activity_at')
    op.drop_column('card_models', 'updated_at')
    op.drop_column('card_models', 'created_at')
    op.drop_column('card_members', 'updated_at')
    op.drop_column('card_lists', 'updated_at')
    op.drop_column('card_attributes', 'updated_at')
    op.drop_constraint(None, 'board_templates', type_='foreignkey')
    op.drop_column('board_templates', 'updated_at')
    op.drop_column('board_templates', 'created_by_id')
    op.drop_column('board_templates', 'created_at')
    ### end Alembic commands ###



"""reindex_attribute_positions

Revision ID: 0262
Revises: 0261
Create Date: 2025-01-06 21:42:07.772751

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '0262'
down_revision = '0261'
branch_labels = None
depends_on = None


def upgrade():
    # Deleting any overloaded positions on project and org attributes (deprecated feature)
    op.execute("""
        DELETE FROM attribute_positions WHERE attribute_positions.attribute_id IN (
            SELECT attributes.id FROM attributes
            WHERE group_id IN (SELECT attribute_group_id FROM projects)
            OR group_id IN (SELECT attribute_group_id FROM organizations)
        )
    """)

    # Reindexing positions for projects with models having overloaded positions
    conn = op.get_bind()
    projects_with_overloads = conn.execute(sa.text("""
        SELECT id, attribute_group_id, slug, (SELECT count(*) FROM attributes WHERE group_id = projects.attribute_group_id)
        FROM projects
        WHERE 0 < (
            SELECT count(*)
            FROM attribute_positions
            JOIN attributes ON attributes.id = attribute_positions.attribute_id
            JOIN card_models ON card_models.attribute_group_id = attributes.group_id
            JOIN project_enabled_models ON project_enabled_models.model_id = card_models.id
            WHERE project_enabled_models.project_id = projects.id AND attribute_positions.namespace = projects.id::text
        )
        ORDER BY projects.id
    """))

    for r in projects_with_overloads:
        # Reindexing positions for the project
        project_id = r[0]
        attr_group_id = r[1]
        containers = [row[0] for row in conn.execute(sa.text("""
            SELECT DISTINCT container FROM attributes WHERE group_id = :group_id
        """), {'group_id': attr_group_id})] + [None, '__top__']
        attr_count = r[3]

        if attr_count == 0:
            # No attributes in project, deleting any overloaded positions
            conn.execute(sa.text("""
                DELETE FROM attribute_positions
                USING attributes, card_models, project_enabled_models
                WHERE attributes.id = attribute_positions.attribute_id
                AND card_models.attribute_group_id = attributes.group_id
                AND project_enabled_models.model_id = card_models.id
                AND project_enabled_models.project_id = :project_id
                AND attribute_positions.namespace = :namespace
            """), {'project_id': project_id, 'namespace': str(project_id)})
            continue

        # Deleting any overloaded positions in non-project containers
        conn.execute(sa.text("""
            DELETE FROM attribute_positions
            USING attributes, card_models, project_enabled_models
            WHERE attributes.id = attribute_positions.attribute_id
            AND card_models.attribute_group_id = attributes.group_id
            AND project_enabled_models.model_id = card_models.id
            AND project_enabled_models.project_id = :project_id
            AND attribute_positions.namespace = :namespace
            AND attribute_positions.container NOT IN :containers
        """), {'project_id': project_id, 'namespace': str(project_id), 'containers': tuple(containers)})

        for container in containers:
            # Reindexing container
            position_map = {}

            attributes = conn.execute(sa.text("""
                SELECT id, position FROM attributes
                WHERE group_id = :group_id AND container = :container
                ORDER BY position
            """), {'group_id': attr_group_id, 'container': container})

            for i, attribute in enumerate(attributes):
                position_map[attribute.position] = i
                conn.execute(sa.text("""
                    UPDATE attributes SET position = :position WHERE id = :id
                """), {'position': i, 'id': attribute.id})

            if len(position_map) == 0:
                # No attributes in container, deleting any overloaded positions
                conn.execute(sa.text("""
                    DELETE FROM attribute_positions
                    USING attributes, card_models, project_enabled_models
                    WHERE attributes.id = attribute_positions.attribute_id
                    AND card_models.attribute_group_id = attributes.group_id
                    AND project_enabled_models.model_id = card_models.id
                    AND project_enabled_models.project_id = :project_id
                    AND attribute_positions.namespace = :namespace                 
                    AND attribute_positions.container = :container
                """), {'project_id': project_id, 'namespace': str(project_id), 'container': container})
            else:
                # Remapping relative positions in overloaded positions
                positions = conn.execute(sa.text("""
                    SELECT attribute_positions.id, attribute_positions.relative_to, attribute_positions.position, attribute_positions.container
                    FROM attribute_positions
                    JOIN attributes ON attributes.id = attribute_positions.attribute_id
                    JOIN card_models ON card_models.attribute_group_id = attributes.group_id
                    JOIN project_enabled_models ON project_enabled_models.model_id = card_models.id
                    WHERE project_enabled_models.project_id = :project_id
                    AND attribute_positions.namespace = :namespace                 
                    AND attribute_positions.container = :container
                    ORDER BY attribute_positions.position
                """), {'project_id': project_id, 'namespace': str(project_id), 'container': container})
                for position in positions:
                    if position.relative_to not in position_map:
                        conn.execute(sa.text("""
                            DELETE FROM attribute_positions WHERE id = :id
                        """), {'id': position.id})
                    else:
                        conn.execute(sa.text("""
                            UPDATE attribute_positions SET relative_to = :relative_to WHERE id = :id
                        """), {'relative_to': position_map[position.relative_to], 'id': position.id})
                            
    # Reindexing positions for project attributes across all projects without overloaded positions
    op.execute("""
        WITH
            projects_without_overloads as (
                SELECT attribute_group_id FROM projects
                WHERE 0 = (
                    SELECT count(*)
                    FROM attribute_positions
                    JOIN attributes ON attributes.id = attribute_positions.attribute_id
                    JOIN card_models ON card_models.attribute_group_id = attributes.group_id
                    JOIN project_enabled_models ON project_enabled_models.model_id = card_models.id
                    WHERE project_enabled_models.project_id = projects.id AND attribute_positions.namespace = projects.id::text
                )
            ),
            positions as (
                SELECT id, row_number() OVER (partition by group_id, container order by position) AS p
                FROM attributes
                JOIN projects_without_overloads ON attributes.group_id = projects_without_overloads.attribute_group_id
            )
        UPDATE attributes SET position = positions.p - 1 FROM positions
        WHERE positions.id = attributes.id
    """)

    # Reindexing all positions for attributes in models
    op.execute("""
        WITH positions as (
            SELECT attributes.id, row_number() OVER (partition by attributes.group_id, attributes.container order by position) AS p
            FROM attributes JOIN card_models ON card_models.attribute_group_id = attributes.group_id
        ) UPDATE attributes SET position = positions.p - 1 FROM positions
        WHERE positions.id = attributes.id
    """)

    # Reindexing all overloaded positions
    op.execute("""
        WITH positions as (
            SELECT ap.id, row_number() OVER (partition by attr.group_id, ap.namespace, ap.container, ap.relative_to order by ap.position) AS p
            FROM attribute_positions ap JOIN attributes attr ON attr.id = ap.attribute_id
        ) UPDATE attribute_positions SET position = positions.p - 1 FROM positions
        WHERE positions.id = attribute_positions.id
    """)

    op.execute("""UPDATE card_models SET cache_id = uuid_generate_v4()""")


def downgrade():
    pass



"""add_cardlists_icon

Revision: 0013
Revision ID: 3ce3a91bc9eb
Revises: 48b6284b245e
Create Date: 2015-07-07 14:16:06.405804

"""

# revision identifiers, used by Alembic.
revision = '3ce3a91bc9eb'
down_revision = '48b6284b245e'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('card_lists', sa.Column('icon', sa.String(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('card_lists', 'icon')
    ### end Alembic commands ###



"""views_org_id

Revision: 0207
Revision ID: 0f6cbf29afd9
Revises: 826f4548b733
Create Date: 2020-10-27 09:36:58.128263

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0f6cbf29afd9'
down_revision = '826f4548b733'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('views', sa.Column('org_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'views', 'organizations', ['org_id'], ['id'], ondelete='CASCADE')

def downgrade():
    op.drop_column('views', 'org_id')



"""added created_at

Revision: 0004
Revision ID: 4ef62b5814cd
Revises: 171c7a75ead7
Create Date: 2015-06-04 21:13:40.477672

"""

# revision identifiers, used by Alembic.
revision = '4ef62b5814cd'
down_revision = '171c7a75ead7'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('card_subscriptions', sa.Column('created_at', sa.DateTime(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('card_subscriptions', 'created_at')
    ### end Alembic commands ###



"""cleanup_duplicate_user_orgs

Revision: 0108
Revision ID: 7583b4a2428
Revises: 3b7c458d8852
Create Date: 2017-11-14 13:41:45.249608

"""

# revision identifiers, used by Alembic.
revision = '7583b4a2428'
down_revision = '3b7c458d8852'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.execute("""
        DELETE FROM organizations WHERE
            is_user_org AND
            id != (select user_org_id from users where users.id = organizations.created_by_id)
    """)


def downgrade():
    pass



"""updated-projects-report-view

Revision ID: 0247
Revises: 0246
Create Date: 2022-10-20 08:23:18.373501

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0247'
down_revision = '0246'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        CREATE OR REPLACE VIEW projects_report_view AS
        SELECT
            p.id,
            p.created_at,
            p.updated_at,
            p.owner_id,
            p.created_by_id,
            p.is_template,
            p.is_archived,
            p.org_members_can_observe,
            p.org_members_can_autojoin,
            p.default_org_member_permission,
            p.import_status,
            p.total_invites,
            p.default_org_member_role_id,
            p.is_public,
            p.send_emails,
            p.allow_anon_inbound_emails,
            p.include_in_template_library,
            p.copied_from_template_id,
            c.last_activity_at,
            (SELECT COUNT(*) FROM project_members WHERE project_id = p.id) nb_members,
            (SELECT COUNT(*) FROM cards WHERE project_id = p.id) nb_cards,
            (SELECT MAX(last_activity_at) FROM cards WHERE project_id = p.id) last_card_activity_at,
            (SELECT COUNT(*) FROM automation_rules WHERE project_id = p.id) nb_automations,
            (SELECT COUNT(*) FROM views WHERE project_id = p.id) nb_views,
            (SELECT COUNT(*) FROM views WHERE project_id = p.id AND mode='form') nb_forms
        FROM projects p JOIN cards c ON c.id = p.top_level_card_id
    """)


def downgrade():
    pass



"""create_request_perm

Revision: 0184
Revision ID: ebacad20699
Revises: 3b4dea6ade9d
Create Date: 2020-05-19 13:26:26.771370

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = 'ebacad20699'
down_revision = '3b4dea6ade9d'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        INSERT INTO role_permissions (role_id, object_type, actions)
        SELECT
            roles.id,
            'request',
            '{"CREATE", "DELETE"}'::varchar[]
        FROM roles WHERE roles.builtin in ('ADMIN', 'MEMBER')
    """)

    op.execute("""
        UPDATE roles SET cache_id = uuid_generate_v4()
        WHERE roles.id IN (
            SELECT DISTINCT roles.id
            FROM roles, role_permissions
            WHERE role_permissions.role_id = roles.id
            AND roles.builtin in ('ADMIN', 'MEMBER')
            AND role_permissions.object_type = 'request'
        )
    """)

    op.execute("""
        UPDATE projects SET members_cache_id = uuid_generate_v4()
    """)


def downgrade():
    pass



"""query_charts_options

Revision ID: 0277
Revises: 0276
Create Date: 2025-08-19 10:25:34.363934

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0277'
down_revision = '0276'
branch_labels = None
depends_on = None


views = sa.Table(
    'views',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('mode', sa.String),
    sa.Column('mode_options', postgresql.JSONB)
)


def upgrade():
    conn = op.get_bind()
    updates = []
    for view in conn.execute(views.select().where(views.c.mode == 'report')):
        if isinstance(view['mode_options'], dict) and view['mode_options'].get('widgets'):
            for widget in view['mode_options']['widgets']:
                if isinstance(widget['options'], list):
                    widget['options'] = {} # catch a previous bug
                if not widget['options'].get('compute') and widget['options'].get('query'):
                    widget['options']['compute'] = "query"
            updates.append({'vid': view['id'], 'opts': view['mode_options']})

    if updates:
        stmt = views.update().where(views.c.id==sa.bindparam('vid')).values(mode_options=sa.bindparam('opts'))
        conn.execute(stmt, updates)


def downgrade():
    pass



"""clear_group_type_card_attributes

Revision ID: 0286
Revises: 0285
Create Date: 2026-03-03 11:09:29.673537

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0286'
down_revision = '0285'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""UPDATE cards SET cache_id = uuid_generate_v4()
        FROM card_attributes
        WHERE card_attributes.card_id = cards.id
        AND card_attributes.type_name = 'group_type'""")
    op.execute("""DELETE FROM card_attributes WHERE type_name = 'group_type'""")


def downgrade():
    pass



"""public_role

Revision: 0121
Revision ID: 3cb0832a6d79
Revises: 5555e993377c
Create Date: 2018-03-30 15:41:27.520322

"""

# revision identifiers, used by Alembic.
revision = '3cb0832a6d79'
down_revision = '5555e993377c'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


orgs = sa.Table(
    'organizations',
    sa.MetaData(),
    sa.Column('id', sa.Integer()),
    sa.Column('default_public_role_id', sa.Integer()),
    sa.Column('default_guest_role_id', sa.Integer())
)

projects = sa.Table(
    'projects',
    sa.MetaData(),
    sa.Column('id', sa.Integer()),
    sa.Column('owner_id', sa.Integer()),
    sa.Column('public_permission', sa.Integer()),
    sa.Column('is_public', sa.Boolean()),
    sa.Column('public_role_id', sa.Integer())
)

roles = sa.Table(
    'roles',
    sa.MetaData(),
    sa.Column('id', sa.Integer()),
    sa.Column('owner_id', sa.Integer()),
    sa.Column('builtin', sa.String())
)

def upgrade():
    conn = op.get_bind()

    op.add_column('organizations', sa.Column('default_public_role_id', sa.Integer(), nullable=True))
    op.add_column('organizations', sa.Column('default_guest_role_id', sa.Integer(), nullable=True))
    op.create_foreign_key('organizations_default_public_role_id_fkey', 'organizations', 'roles', ['default_public_role_id'], ['id'], ondelete='SET NULL')
    op.create_foreign_key('organizations_default_guest_role_id_fkey', 'organizations', 'roles', ['default_guest_role_id'], ['id'], ondelete='SET NULL')

    op.add_column('projects', sa.Column('is_public', sa.Boolean(), nullable=True))
    op.add_column('projects', sa.Column('public_role_id', sa.Integer(), nullable=True))
    op.create_foreign_key('projects_public_role_id_fkey', 'projects', 'roles', ['public_role_id'], ['id'], ondelete='SET NULL')

    for org in conn.execute(orgs.select()):
        role = conn.execute(roles.select().where(sa.and_(roles.c.owner_id == org.id, roles.c.builtin == 'OBSERVER'))).first()
        conn.execute(orgs.update().where(orgs.c.id == org.id).values(
            default_public_role_id=role.id if role else None,
            default_guest_role_id=role.id if role else None))

    conn.execute(projects.update().where(projects.c.public_permission==0).values(is_public=False))
    for project in conn.execute(projects.select().where(projects.c.public_permission > 0)):
        builtin = 'OBSERVER'
        if project.public_permission == 2:
            builtin = 'OBSERVER ALLOWED TO COMMENT'
        elif project.public_permission == 3:
            builtin = 'MEMBER'
        role = conn.execute(roles.select().where(sa.and_(roles.c.owner_id == project.owner_id, roles.c.builtin == builtin))).first()
        conn.execute(projects.update().where(projects.c.id == project.id).values(is_public=True, public_role_id=role.id if role else None))


def downgrade():
    op.drop_column('projects', 'public_role_id')
    op.drop_column('projects', 'is_public')



"""add_github_scope

Revision: 0016
Revision ID: 302c2c3a9770
Revises: 1993ef6216c
Create Date: 2015-08-11 15:30:56.922506

"""

# revision identifiers, used by Alembic.
revision = '302c2c3a9770'
down_revision = '1993ef6216c'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('users', sa.Column('github_scope', sa.String(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('users', 'github_scope')
    ### end Alembic commands ###



"""models refactor

Revision: 0034
Revision ID: 2a46d7b96e63
Revises: 166735e0e8f6
Create Date: 2015-11-19 13:42:02.468777

"""

# revision identifiers, used by Alembic.
revision = '2a46d7b96e63'
down_revision = '166735e0e8f6'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    op.add_column('card_models', sa.Column('dynamic_extends', sa.String(), nullable=True))
    op.add_column('card_models', sa.Column('is_base', sa.String(), nullable=True))
    op.execute("UPDATE card_models SET is_base = 'org' WHERE name in ('User Base Model', 'Organization Base Model')")
    op.execute("UPDATE card_models SET is_base = 'project', listable = false WHERE name LIKE '%base model' AND available_for_all is false AND created_in_id IS NOT NULL")
    op.alter_column('card_models', 'available_for_all', new_column_name='auto_enabled_on_new_projects')
    op.add_column('cards', sa.Column('base_card_model_id', sa.Integer(), nullable=True))
    op.create_foreign_key('cards_base_card_model_id', 'cards', 'card_models', ['base_card_model_id'], ['id'], ondelete='SET NULL')
    op.execute("UPDATE cards SET base_card_model_id = model_id WHERE parent_id IS NULL")


def downgrade():
    op.drop_constraint('cards_base_card_model_id', 'cards', type_='foreignkey')
    op.drop_column('cards', 'base_card_model_id')
    op.alter_column('card_models', 'auto_enabled_on_new_projects', new_column_name='available_for_all')
    op.drop_column('card_models', 'is_base')
    op.drop_column('card_models', 'dynamic_extends')



"""delete_null_values_from_card_attributes

Revision ID: 0249
Revises: 0248
Create Date: 2023-02-09 16:43:24.145429

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0249'
down_revision = '0248'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        DELETE FROM card_attributes
        WHERE value IS NULL AND unindexed_value IS NULL
    """)


def downgrade():
    pass



"""org default template

Revision: 0070
Revision ID: 8a9754acb4a
Revises: 423db8281cbe
Create Date: 2016-08-11 16:35:33.251306

"""

# revision identifiers, used by Alembic.
revision = '8a9754acb4a'
down_revision = '423db8281cbe'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    op.add_column('organizations', sa.Column('default_template_id', sa.Integer(), nullable=True))
    op.create_foreign_key('organizations_default_template_id', 'organizations', 'projects', ['default_template_id'], ['id'], ondelete='SET NULL')


def downgrade():
    op.drop_constraint('organizations_default_template_id', 'organizations', type_='foreignkey')
    op.drop_column('organizations', 'default_template_id')



"""automations

Revision: 0219
Revision ID: 233b7875868c
Revises: 681ff6dac033
Create Date: 2021-06-15 12:06:16.718318

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '233b7875868c'
down_revision = '681ff6dac033'
branch_labels = None
depends_on = None


def upgrade():
    op.create_table('automation_rules',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('project_id', sa.Integer(), nullable=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('name', sa.String(), nullable=False),
        sa.Column('description', sa.String(), nullable=True),
        sa.Column('trigger_event_name', sa.String(), nullable=False),
        sa.Column('trigger_options', postgresql.JSONB(), nullable=True),
        sa.Column('sequence', postgresql.JSON(), nullable=True),
        sa.Column('active', sa.Boolean(), nullable=False),
        sa.Column('parallel', sa.Boolean(), nullable=False),
        sa.Column('last_trigger_at', sa.DateTime(), nullable=True),
        sa.Column('trigger_count_total', sa.Integer(), nullable=False),
        sa.Column('trigger_count_period', sa.Integer(), nullable=False),
        sa.Column('run_token', sa.Text(), nullable=False),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['project_id'], ['projects.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_automation_rules_project_id'), 'automation_rules', ['project_id'], unique=False)
    op.create_index(op.f('ix_automation_rules_active'), 'automation_rules', ['active'], unique=False)
    op.create_index(op.f('ix_automation_rules_trigger_event_name'), 'automation_rules', ['trigger_event_name'], unique=False)
    op.create_index(op.f('ix_automation_rules_trigger_options'), 'automation_rules', ['trigger_options'], unique=False)
    op.create_index(op.f('ix_automation_rules_run_token'), 'automation_rules', ['run_token'], unique=True)

    op.create_table('automation_states',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('rule_id', sa.Integer(), nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('namespace', sa.String(), nullable=True),
        sa.Column('step', sa.String(), nullable=True),
        sa.Column('data', postgresql.JSON(), nullable=True),
        sa.Column('error', sa.String(), nullable=True),
        sa.ForeignKeyConstraint(['rule_id'], ['automation_rules.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_automation_states_rule_id'), 'automation_states', ['rule_id'], unique=False)
    op.create_index(op.f('ix_automation_states_namespace'), 'automation_states', ['namespace'], unique=False)
    op.create_index(op.f('ix_automation_states_error_updated_at'), 'automation_states', ['error', 'updated_at'], unique=False)


def downgrade():
    op.drop_table('automation_states')
    op.drop_table('automation_rules')



"""email_not_unique

Revision: 0061
Revision ID: bfc8eb1f8c2
Revises: 7a809a8d04e
Create Date: 2016-08-05 13:58:25.895086

"""

# revision identifiers, used by Alembic.
revision = 'bfc8eb1f8c2'
down_revision = '7a809a8d04e'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.drop_constraint('users_email_key', 'users')
    op.create_index(op.f('ix_users_email'), 'users', ['email'], unique=False)


def downgrade():
    op.drop_index(op.f('ix_users_email'), table_name='users')
    op.create_unique_constraint("users_email_key", "users", ["email"])



"""users_report_view

Revision: 0216
Revision ID: 84adae4688a7
Revises: 3c258d81bd0c
Create Date: 2021-06-21 11:55:30.611082

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '84adae4688a7'
down_revision = '3c258d81bd0c'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        CREATE OR REPLACE VIEW users_report_view AS
        SELECT users.id,
            users.signup_at,
            users.signup_from,
            users.signup_provider,
            users.auth_providers,
            users.last_login_at,
            users.last_login_from,
            users.last_login_provider,
            split_part(users.email::text, '@'::text, 2) AS email_domain,
            users.can_create_organizations,
            users.locale,
            users.timezone,
            users.enable_email_notifications,
            users.updated_at,
            users.is_admin,
            users.subscription_required,
            users.trial_ends_at,
            users.can_join_organizations,
            users.is_observer,
            users.is_disabled,
            users.disabled_at,
            users.last_activity_at,
            users.onboarded,
            users.last_password_change_at,
            users.last_terms_validation,
            users.email_newsletter,
            users.email_notifications,
            users.is_super_admin,
            users.signup_country,
            users.signup_cpc_source,
            users.signup_cpc_campaign,
            users.signup_cpc_ad,
            users.signup_cpc_keyword,
            users.signup_cpc_sessionid,
            users.last_access_at,
            users.last_password_change_from,
            users.is_auto_disabled,
            users.two_factor_auth_enabled,
            users.email_validated,
            users.email_validated_at
        FROM users;
    """)


def downgrade():
    pass



"""requests

Revision: 0077
Revision ID: f6be47c2049
Revises: 3610446b33b5
Create Date: 2016-11-06 15:35:30.557212

"""

# revision identifiers, used by Alembic.
revision = 'f6be47c2049'
down_revision = '3610446b33b5'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    op.create_table('request_forms',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('project_id', sa.Integer(), nullable=True),
        sa.Column('name', sa.String(), nullable=True),
        sa.Column('description', sa.Text(), nullable=True),
        sa.Column('email_message', sa.Text(), nullable=True),
        sa.Column('slug', sa.String(), nullable=True),
        sa.Column('public_permission', sa.Integer(), nullable=True),
        sa.Column('allow_anonymous', sa.Boolean(), nullable=True),
        sa.Column('allow_comments', sa.Boolean(), nullable=True),
        sa.Column('tracking_group_type_id', sa.Integer(), nullable=True),
        sa.Column('add_card_to_group_id', sa.Integer(), nullable=True),
        sa.Column('ask_title', sa.Boolean(), nullable=True),
        sa.Column('title_label', sa.String(), nullable=True),
        sa.Column('card_model_id', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['add_card_to_group_id'], ['card_groups.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['card_model_id'], ['card_models.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['project_id'], ['projects.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['tracking_group_type_id'], ['card_group_types.id'], ondelete='SET NULL'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_request_forms_project_id'), 'request_forms', ['project_id'], unique=False)
    op.create_index(op.f('ix_request_forms_slug'), 'request_forms', ['slug'], unique=False)

    op.create_table('request_form_tracking_groups',
        sa.Column('request_form_id', sa.Integer(), nullable=True),
        sa.Column('group_id', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['group_id'], ['card_groups.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['request_form_id'], ['request_forms.id'], ondelete='CASCADE')
    )
    op.create_index(op.f('ix_request_form_tracking_groups_group_id'), 'request_form_tracking_groups', ['group_id'], unique=False)
    op.create_index(op.f('ix_request_form_tracking_groups_request_form_id'), 'request_form_tracking_groups', ['request_form_id'], unique=False)

    op.create_table('request_form_attributes',
        sa.Column('request_form_id', sa.Integer(), nullable=True),
        sa.Column('attribute_id', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['attribute_id'], ['attributes.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['request_form_id'], ['request_forms.id'], ondelete='CASCADE')
    )
    op.create_index(op.f('ix_request_form_attributes_attribute_id'), 'request_form_attributes', ['attribute_id'], unique=False)
    op.create_index(op.f('ix_request_form_attributes_request_form_id'), 'request_form_attributes', ['request_form_id'], unique=False)
    
    op.create_table('requests',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('request_form_id', sa.Integer(), nullable=True),
        sa.Column('user_id', sa.Integer(), nullable=True),
        sa.Column('email', sa.String(), nullable=True),
        sa.Column('token', sa.String(), nullable=True),
        sa.Column('card_id', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['card_id'], ['cards.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['request_form_id'], ['request_forms.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['user_id'], ['users.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_requests_request_form_id'), 'requests', ['request_form_id'], unique=False)
    op.create_index(op.f('ix_requests_token'), 'requests', ['token'], unique=False)
    op.create_index(op.f('ix_requests_card_id'), 'requests', ['card_id'], unique=False)
    op.create_index(op.f('ix_requests_user_id'), 'requests', ['user_id'], unique=False)

    op.add_column('card_comments', sa.Column('show_in_request', sa.Boolean, nullable=True))
    op.create_index(op.f('ix_card_comments_show_in_request'), 'card_comments', ['show_in_request'], unique=False)
    op.execute("UPDATE card_comments SET show_in_request = false")


def downgrade():
    op.drop_index(op.f('ix_requests_request_form_id'), table_name='requests')
    op.drop_index(op.f('ix_requests_token'), table_name='requests')
    op.drop_index(op.f('ix_requests_card_id'), table_name='requests')
    op.drop_index(op.f('ix_requests_user_id'), table_name='requests')
    op.drop_table('requests')
    op.drop_index(op.f('ix_request_form_attributes_request_form_id'), table_name='request_form_attributes')
    op.drop_index(op.f('ix_request_form_attributes_attribute_id'), table_name='request_form_attributes')
    op.drop_table('request_form_attributes')
    op.drop_index(op.f('ix_request_forms_slug'), table_name='request_forms')
    op.drop_index(op.f('ix_request_forms_project_id'), table_name='request_forms')
    op.drop_table('request_forms')



"""group_type_attribute

Revision: 0067
Revision ID: 31f84a385d20
Revises: 16627338400e
Create Date: 2016-07-21 20:44:49.279036

"""

# revision identifiers, used by Alembic.
revision = '31f84a385d20'
down_revision = '16627338400e'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

organizations = sa.Table(
    'organizations',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('default_card_model_id', sa.Integer),
)

attribute_groups = sa.Table(
    'attribute_groups',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer)
)

attributes = sa.Table(
    'attributes',
    sa.MetaData(),
    sa.Column('id', sa.Integer()),
    sa.Column('card_model_id', sa.Integer()),
    sa.Column('name', sa.String()),
    sa.Column('type_name', sa.String()),
    sa.Column('options', sa.PickleType()),
    sa.Column('group_id', sa.Integer())
)

card_attributes = sa.Table(
    'card_attributes',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('attribute_id', sa.Integer)
)

group_types = sa.Table(
    'card_group_types',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer),
    sa.Column('name', sa.String()),
    sa.Column('attribute_id', sa.Integer)
)

card_enabled_group_types = sa.Table(
    'card_enabled_group_types',
    sa.MetaData(),
    sa.Column('group_type_id', sa.Integer()),
    sa.Column('card_id', sa.Integer())
)

card_models = sa.Table(
    'card_models',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer),
    sa.Column('is_base', sa.String),
    sa.Column('created_in_id', sa.Integer),
    sa.Column('is_project_specific', sa.String),
    sa.Column('attribute_group_id', sa.Integer),
    sa.Column('extends_id', sa.Integer),
    sa.Column('dynamic_extends', sa.String)
)


def upgrade():
    op.add_column('card_group_types', sa.Column('attribute_id', sa.Integer(), nullable=True))
    op.create_index(op.f('ix_card_group_types_attribute_id'), 'card_group_types', ['attribute_id'], unique=False)

    conn = op.get_bind()
    updates = []
    deletes = []

    grouptypes = {}

    q = attributes.select().where(attributes.c.type_name == 'group_type')
    for attr in conn.execute(q):
        if attr['options'].get('group_type'):
            gt = conn.execute(group_types.select().where(group_types.c.id==attr['options']['group_type'])).first()
            if gt and gt['name'] not in ('Lists', 'Milestones'):
                if gt.id not in grouptypes:
                    org_model = conn.execute(card_models.join(organizations, card_models.c.id==organizations.c.default_card_model_id).select().where(organizations.c.id==gt.owner_id)).first()
                    conn.execute(attributes.update().where(attributes.c.id == attr.id).values(
                        group_id = org_model.attribute_group_id,
                        card_model_id = org_model[0],
                        name=gt.name
                    ))
                    grouptypes[gt.id] = attr.id
                    updates.append({'gt_id': gt['id'], 'attr_id': attr['id']})
                    continue
                else:
                    conn.execute(card_attributes.update().where(card_attributes.c.attribute_id==attr.id).values(
                        attribute_id = grouptypes[gt.id]
                    ))
        deletes.append({'attr_id': attr['id']})

    if updates:
        stmt = group_types.update().where(group_types.c.id==sa.bindparam('gt_id')).values(
            attribute_id=sa.bindparam('attr_id'))
        conn.execute(stmt, updates)

    if deletes:
        stmt = attributes.delete().where(attributes.c.id == sa.bindparam('attr_id'))
        conn.execute(stmt, deletes)

    if updates:
        stmt = card_enabled_group_types.delete().where(card_enabled_group_types.c.group_type_id == sa.bindparam('gt_id'))
        conn.execute(stmt, updates)


def downgrade():
    op.drop_index(op.f('ix_card_group_types_attribute_id'), table_name='card_group_types')
    op.drop_column('card_group_types', 'attribute_id')



"""finished_at

Revision: 0054
Revision ID: 4b225430041e
Revises: 78a2be497a8
Create Date: 2016-03-21 18:03:16.448997

"""

# revision identifiers, used by Alembic.
revision = '4b225430041e'
down_revision = '78a2be497a8'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('cards', sa.Column('finished_at', sa.DateTime(), nullable=True))
    op.create_index(op.f('ix_cards_finished_at'), 'cards', ['finished_at'], unique=False)
    op.execute("""
        update cards set finished_at = (select min(created_at) from card_change_history
        where state in ('completed', 'closed', 'dropped') and card_id = cards.id)
    """)


def downgrade():
    op.drop_index(op.f('ix_cards_finished_at'), table_name='cards')
    op.drop_column('cards', 'finished_at')



"""clear_cache_private_projects

Revision: 0124
Revision ID: 76df933e43b
Revises: 51c94ea54825
Create Date: 2018-04-11 11:44:58.805986

"""

# revision identifiers, used by Alembic.
revision = '76df933e43b'
down_revision = '51c94ea54825'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    conn = op.get_bind()
    conn.execute('UPDATE projects SET members_cache_id = uuid_generate_v4() FROM organizations o WHERE o.id = projects.owner_id AND o.is_user_org')


def downgrade():
    pass



# -*- coding: utf-8 -*-
"""advanced_permissions

Revision: 0120
Revision ID: 5555e993377c
Revises: 537252d4f3d3
Create Date: 2017-01-31 15:23:33.777168

"""

# revision identifiers, used by Alembic.
revision = '5555e993377c'
down_revision = '537252d4f3d3'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    conn = op.get_bind()

    op.create_table('roles',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('owner_id', sa.Integer(), nullable=True),
        sa.Column('name', sa.String(), nullable=True),
        sa.Column('description', sa.Text(), nullable=True),
        sa.Column('cache_id', sa.String(), nullable=True),
        sa.Column('builtin', sa.String(), nullable=True),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['owner_id'], ['organizations.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_roles_owner_id'), 'roles', ['owner_id'], unique=False)

    role_permissions = op.create_table('role_permissions',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('role_id', sa.Integer(), nullable=True),
        sa.Column('project_id', sa.Integer(), nullable=True),
        sa.Column('object_type', sa.String(), nullable=True),
        sa.Column('object_id', sa.Integer(), nullable=True),
        sa.Column('actions', postgresql.ARRAY(sa.String()), nullable=False),
        sa.ForeignKeyConstraint(['project_id'], ['projects.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['role_id'], ['roles.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_role_permissions_project_id'), 'role_permissions', ['project_id'], unique=False)
    op.create_index(op.f('ix_role_permissions_role_id'), 'role_permissions', ['role_id'], unique=False)

    op.add_column('project_members', sa.Column('role_id', sa.Integer(), nullable=True))
    op.create_index(op.f('ix_project_members_role_id'), 'project_members', ['role_id'], unique=False)
    op.create_foreign_key('project_members_role_id_fkey', 'project_members', 'roles', ['role_id'], ['id'], ondelete='CASCADE')

    op.add_column('organizations', sa.Column('default_org_member_role_id', sa.Integer(), nullable=True))
    op.create_foreign_key('default_org_member_role_id_fkey', 'organizations', 'roles', ['default_org_member_role_id'], ['id'], ondelete='SET NULL')

    op.add_column('projects', sa.Column('default_org_member_role_id', sa.Integer(), nullable=True))
    op.create_foreign_key('default_org_member_role_id_fkey', 'projects', 'roles', ['default_org_member_role_id'], ['id'], ondelete='SET NULL')

    # ADMIN
    r = conn.execute("""
        INSERT INTO roles (created_at, created_by_id, owner_id, name, description, builtin, cache_id)
        SELECT now(), o.created_by_id, o.id, 'Admin', 'Member who can manage project settings', 'ADMIN', uuid_generate_v4()
        FROM organizations o, users u WHERE o.created_by_id = u.id AND (u.locale != 'fr' OR u.locale IS NULL) RETURNING id
    """)
    r2 = conn.execute("""
        INSERT INTO roles (created_at, created_by_id, owner_id, name, description, builtin, cache_id)
        SELECT now(), o.created_by_id, o.id, 'Admin', 'Membre qui peut gérer les paramètres du projet', 'ADMIN', uuid_generate_v4()
        FROM organizations o, users u WHERE o.created_by_id = u.id AND u.locale = 'fr' RETURNING id
    """)

    perms = []
    for r_infos in (r, r2):
        for r_info in r_infos:
            role_id = r_info[0]
            perms.extend([
                {'role_id': role_id, 'object_type': 'project', 'actions': ['MANAGE_SUB_BOARDS','OPTIONS','ACCESS','MANAGE_ATTRIBUTES','MANAGE_SHARED_VIEWS','MANAGE_RECURRING']},
                {'role_id': role_id, 'object_type': 'card', 'actions': ['CREATE','EDIT','DELETE','SWITCH_MODEL','ARCHIVE']},
                {'role_id': role_id, 'object_type': 'card_model', 'actions': ['CREATE','EDIT','DELETE']},
                {'role_id': role_id, 'object_type': 'card_group_type', 'actions': ['CREATE','EDIT','DELETE']},
                {'role_id': role_id, 'object_type': 'card_log_type', 'actions': ['CREATE','EDIT','DELETE']},
                {'role_id': role_id, 'object_type': 'card_group', 'actions': ['CREATE','EDIT','MOVE_CARDS','DELETE']},
                {'role_id': role_id, 'object_type': 'request_form', 'actions': ['CREATE','EDIT','DELETE']},
                {'role_id': role_id, 'object_type': 'card_comment', 'actions': ['CREATE','EDIT','DELETE','DELETE_MINE']},
                {'role_id': role_id, 'object_type': 'card_log', 'actions': ['CREATE','EDIT','DELETE','DELETE_MINE']},
                {'role_id': role_id, 'object_type': 'attribute', 'actions': ['EDIT']}
            ])
    op.bulk_insert(role_permissions, perms)

    # MEMBER
    r = conn.execute("""
        INSERT INTO roles (created_at, created_by_id, owner_id, name, description, builtin, cache_id)
        SELECT now(), o.created_by_id, o.id, 'Member', 'Create, edit and collaborate with cards and groups', 'MEMBER', uuid_generate_v4()
        FROM organizations o, users u WHERE o.created_by_id = u.id AND (u.locale != 'fr' OR u.locale IS NULL) RETURNING id
    """)
    r2 = conn.execute("""
        INSERT INTO roles (created_at, created_by_id, owner_id, name, description, builtin, cache_id)
        SELECT now(), o.created_by_id, o.id, 'Membre', 'Peut créer, modifier et collaborer sur les cartes et les groupes', 'MEMBER', uuid_generate_v4()
        FROM organizations o, users u WHERE o.created_by_id = u.id AND u.locale = 'fr' RETURNING id
    """)
    perms = []
    for r_infos in (r, r2):
        for r_info in r_infos:
            role_id = r_info[0]
            perms.extend([
                {'role_id': role_id, 'object_type': 'card', 'actions': ['CREATE','EDIT','DELETE','SWITCH_MODEL','ARCHIVE']},
                {'role_id': role_id, 'object_type': 'attribute', 'actions': ['EDIT']},
                {'role_id': role_id, 'object_type': 'card_group', 'actions': ['MOVE_CARDS']},
                {'role_id': role_id, 'object_type': 'card_comment', 'actions': ['CREATE','DELETE_MINE']},
                {'role_id': role_id, 'object_type': 'card_log', 'actions': ['CREATE','DELETE_MINE']}
            ])
    op.bulk_insert(role_permissions, perms)
    conn.execute("UPDATE organizations o SET default_org_member_role_id = (SELECT id FROM roles WHERE owner_id = o.id AND builtin = 'MEMBER')")

    # OBSERVER CAN COMMENT
    r = conn.execute("""
        INSERT INTO roles (created_at, created_by_id, owner_id, name, description, builtin, cache_id)
        SELECT now(), o.created_by_id, o.id, 'Observer allowed to comment', 'Read-only access but can comment on cards', 'OBSERVER ALLOWED TO COMMENT', uuid_generate_v4()
        FROM organizations o, users u WHERE o.created_by_id = u.id AND (u.locale != 'fr' OR u.locale IS NULL) RETURNING id
    """)
    r2 = conn.execute("""
        INSERT INTO roles (created_at, created_by_id, owner_id, name, description, builtin, cache_id)
        SELECT now(), o.created_by_id, o.id, 'Observateur autorisé à commenter', 'Accès en lecture seule mais peut commenter', 'OBSERVER ALLOWED TO COMMENT', uuid_generate_v4()
        FROM organizations o, users u WHERE o.created_by_id = u.id AND u.locale = 'fr' RETURNING id
    """)
    perms = []
    for r_infos in (r, r2):
        for r_info in r_infos:
            role_id = r_info[0]
            perms.append({'role_id': role_id, 'object_type': 'card_comment', 'actions': ['CREATE','DELETE_MINE']})
    op.bulk_insert(role_permissions, perms)

    #OBSERVER
    r = conn.execute("""
        INSERT INTO roles (created_at, created_by_id, owner_id, name, description, builtin, cache_id)
        SELECT now(), o.created_by_id, o.id, 'Observer', 'Read-only access', 'OBSERVER', uuid_generate_v4()
        FROM organizations o, users u WHERE o.created_by_id = u.id AND (u.locale != 'fr' OR u.locale IS NULL) RETURNING id
    """)
    r2 = conn.execute("""
        INSERT INTO roles (created_at, created_by_id, owner_id, name, description, builtin, cache_id)
        SELECT now(), o.created_by_id, o.id, 'Observateur', 'Accès en lecture seule', 'OBSERVER', uuid_generate_v4()
        FROM organizations o, users u WHERE o.created_by_id = u.id AND u.locale = 'fr' RETURNING id
    """)

    # Set default org member role

    conn.execute("""UPDATE projects SET default_org_member_role_id = (
        SELECT id FROM roles WHERE owner_id = projects.owner_id AND builtin = 'ADMIN'
    ) WHERE default_org_member_permission = 4""")

    conn.execute("""UPDATE projects SET default_org_member_role_id = (
        SELECT id FROM roles WHERE owner_id = projects.owner_id AND builtin = 'MEMBER'
    ) WHERE default_org_member_permission = 3""")

    conn.execute("""UPDATE projects SET default_org_member_role_id = (
        SELECT id FROM roles WHERE owner_id = projects.owner_id AND builtin = 'OBSERVER ALLOWED TO COMMENT'
    ) WHERE default_org_member_permission = 2""")

    conn.execute("""UPDATE projects SET default_org_member_role_id = (
        SELECT id FROM roles WHERE owner_id = projects.owner_id AND builtin = 'OBSERVER'
    ) WHERE default_org_member_permission = 1""")

    # update member roles

    conn.execute("""UPDATE project_members SET role_id = (
        SELECT roles.id FROM roles, projects WHERE project_members.project_id = projects.id
        AND roles.owner_id = projects.owner_id AND roles.builtin = 'ADMIN'
    ) WHERE permission = 4""")

    conn.execute("""UPDATE project_members SET role_id = (
        SELECT roles.id FROM roles, projects WHERE project_members.project_id = projects.id
        AND roles.owner_id = projects.owner_id AND roles.builtin = 'MEMBER'
    ) WHERE permission = 3""")

    conn.execute("""UPDATE project_members SET role_id = (
        SELECT roles.id FROM roles, projects WHERE project_members.project_id = projects.id
        AND roles.owner_id = projects.owner_id AND roles.builtin = 'OBSERVER ALLOWED TO COMMENT'
    ) WHERE permission = 2""")

    conn.execute("""UPDATE project_members SET role_id = (
        SELECT roles.id FROM roles, projects WHERE project_members.project_id = projects.id
        AND roles.owner_id = projects.owner_id AND roles.builtin = 'OBSERVER'
    ) WHERE permission = 1""")

    conn.execute('UPDATE organizations SET cache_id = uuid_generate_v4()')
    conn.execute('UPDATE projects SET members_cache_id = uuid_generate_v4()')

def downgrade():
    op.drop_constraint('default_org_member_role_id_fkey', 'projects', type_='foreignkey')
    op.drop_column('projects', 'default_org_member_role_id')

    op.drop_constraint('project_members_role_id_fkey', 'project_members', type_='foreignkey')
    op.drop_index(op.f('ix_project_members_role_id'), table_name='project_members')
    op.drop_column('project_members', 'role_id')

    op.drop_index(op.f('ix_role_permissions_role_id'), table_name='role_permissions')
    op.drop_index(op.f('ix_role_permissions_project_id'), table_name='role_permissions')
    op.drop_table('role_permissions')
    op.drop_index(op.f('ix_roles_owner_id'), table_name='roles')
    op.drop_table('roles')
